/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.contact.dao;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;

import sync4j.exchange.httptransport.WebDavHttpTransport;
import sync4j.exchange.items.contact.model.Contact;
import sync4j.exchange.items.common.dao.ItemDAO;
import sync4j.exchange.xml.XmlParseException;
import sync4j.exchange.xml.XmlParser;

import sync4j.exchange.util.StringTools;

import sync4j.exchange.DataAccessException;

import sync4j.foundation.pdi.contact.Address;
import sync4j.foundation.pdi.contact.BusinessDetail;
import sync4j.foundation.pdi.contact.ContactDetail;
import sync4j.foundation.pdi.contact.Email;
import sync4j.foundation.pdi.contact.Name;
import sync4j.foundation.pdi.contact.Note;
import sync4j.foundation.pdi.contact.PersonalDetail;
import sync4j.foundation.pdi.contact.Note;
import sync4j.foundation.pdi.contact.Phone;
import sync4j.foundation.pdi.contact.Title;
import sync4j.foundation.pdi.contact.WebPage;

import sync4j.framework.server.store.NotFoundException;

/*
 * This class implements methods to access contacts data
 * in exchange server datastore
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: ContactDAO.java,v 1.22 2005/06/27 15:08:53 fabius Exp $
 *
 **/
public class ContactDAO extends ItemDAO {

    //---------------------------------------------------------------- Constants

    private static final String TAG_ANNIVERSARY
        = "weddinganniversary"                                             ;
    private static final String TAG_BIRTHDAY          = "bday"             ;
    private static final String TAG_CATEGORIES        = "keywordsutf8"     ;
    private static final String TAG_CHILDREN          = "childrensnames"   ;
    private static final String TAG_DEPARTMENT        = "department"       ;
    private static final String TAG_FILEAS            = "fileas"           ;
    public  static final String TAG_FIRSTNAME         = "givenname"        ;
    private static final String TAG_HOBBY             = "hobbies"          ;
    private static final String TAG_HOME_CITY         = "homecity"         ;
    private static final String TAG_HOME_COUNTRY      = "homecountry"      ;
    private static final String TAG_HOME_POST_OFFICE_BOX
        ="homepostofficebox"                                               ;
    private static final String TAG_HOME_STATE        = "homestate"        ;
    private static final String TAG_HOME_POSTAL_CODE  = "homepostalcode"   ;
    private static final String TAG_HOME_STREET       = "homestreet"       ;
    private static final String TAG_IMPORTANCE        = "importance"       ;
    private static final String TAG_INITIALS          = "initials"         ;
    private static final String TAG_JOBTITLE          = "title"            ;
    private static final String TAG_LANGUAGE          = "language"         ;
    public  static final String TAG_LASTNAME          = "sn"        ;
    public  static final String TAG_MIDDLENAME        = "middlename"       ;
    private static final String TAG_MILEAGE           = "mileage"          ;
    private static final String TAG_SUFFIX            = "namesuffix"       ;
    private static final String TAG_NICKNAME          = "nickname"         ;
    private static final String TAG_OFFICE_LOCATION   = "roomnumber"       ;
    private static final String TAG_ORGANIZATION      = "organization"     ;
    private static final String TAG_ORGANIZATION_ID_NUMBER
        = "employeenumber"                                                 ;
    private static final String TAG_OTHER_CITY        = "othercity"        ;
    private static final String TAG_OTHER_COUNTRY     = "othercountry"     ;
    private static final String TAG_OTHER_POSTALCODE  = "otherpostalcode"  ;
    private static final String TAG_OTHER_POST_OFFICE_BOX
        = "otherpostofficebox"                                             ;
    private static final String TAG_OTHER_STATE       = "otherstate"       ;
    private static final String TAG_OTHER_STREET      = "otherstreet"      ;
    private static final String TAG_SECRETARY         = "secretarycn"      ;
    private static final String TAG_SENSITIVITY       = "sensitivity"      ;
    private static final String TAG_SPOUSECN          = "spousecn"         ;
    private static final String TAG_TITLE             = "personaltitle"    ;
    private static final String TAG_WORK_CITY         = "city"             ;
    private static final String TAG_WORK_COUNTRY      = "country"          ;
    private static final String TAG_WORK_POSTALCODE   = "postalcode"       ;
    private static final String TAG_WORK_POST_OFFICE_BOX
        = "postofficebox"                                                  ;
    private static final String TAG_WORK_STATE        = "state"            ;
    private static final String TAG_WORK_STREET       = "street"           ;

    private static final String
        TAG_YOMI_COMPANY_NAME                         = "yomiorganization" ;
    private static final String TAG_YOMI_FIRST_NAME   = "yomilastname"     ;
    private static final String TAG_YOMI_LAST_NAME    = "yomifirstname"    ;

    private static final String TAG_CALL_BACK_PHONE   = "callbackphone"    ;
    private static final String
            TAG_FAC_SIMILE_TELEPHONE_NUMBER
            = "facsimiletelephonenumber"                                   ;
    private static final String TAG_HOME_FAX          = "homefax"          ;
    private static final String TAG_HOME_PHONE        = "homephone"        ;
    private static final String TAG_HOME_PHONE2       = "homephone2"       ;
    private static final String TAG_ISDN_NUMBER
        = "internationalisdnnumber"                                        ;
    private static final String TAG_MANAGER           = "manager"          ;
    private static final String TAG_MOBILE            = "mobile"           ;
    private static final String
           TAG_ORGANIZATION_MAIN_PHONE
           = "organizationmainphone"                                       ;
    private static final String TAG_OTHER_FAX         = "otherfax"         ;
    private static final String TAG_OTHER_MOBILE      = "othermobile"      ;
    private static final String TAG_OTHER_TELEPHONE
           = "othertelephone"                                              ;
    private static final String TAG_PAGER             = "pager"            ;
    private static final String TAG_PROFESSION        = "profession"       ;
    private static final String TAG_SECRETARY_PHONE   = "secretaryphone"   ;
    private static final String TAG_TELEPHONE_NUMBER  = "telephonenumber"  ;
    private static final String TAG_TELEPHONE_NUMBER2 = "telephonenumber2" ;
    private static final String TAG_TELEX_NUMBER      = "telexnumber"      ;

    private static final String TAG_COMPUTER_NETWORK_NAME
        = "computernetworkname"                                            ;

    private static final String
        TAG_BUSINESS_HOME_PAGE
        = "businesshomepage"                                               ;
    private static final String
            TAG_PERSONAL_HOME_PAGE
        = "personalHomePage"                                               ;

    public  static final String TAG_EMAIL1            = "email1"           ;
    private static final String TAG_EMAIL2            = "email2"           ;
    private static final String TAG_EMAIL3            = "email3"           ;

    private static final String TAG_EMAIL1_ADDRESS_TYPE
        = "email1addrtype"                                                 ;
    private static final String TAG_EMAIL2_ADDRESS_TYPE
        = "email2addrtype"                                                 ;
    private static final String TAG_EMAIL3_ADDRESS_TYPE
        = "email3addrtype"                                                 ;

    private static final String TAG_RESPONSE          = "a:response"       ;
    private static final String TAG_REPLUID           = "repluid"          ;
    private static final String TAG_LAST_MODIFIED     = "getlastmodified"  ;


    private static final String WEBDAV_MSG_SELECT_CONTACTS                   =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                          +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                  +
    "<D:sql>\n"                                                              +
    "Select "                                                                +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "            +
    "\"urn:schemas:contacts:weddinganniversary\" AS "
        + TAG_ANNIVERSARY                                                    + ", " +
    "\"urn:schemas:contacts:bday\" AS "             + TAG_BIRTHDAY           + ", " +
    "\"urn:schemas:contacts:childrensnames\" AS "   + TAG_CHILDREN           + ", " +
    "\"urn:schemas:contacts:fileas\" AS "           + TAG_FILEAS             + ", " +
    "\"urn:schemas:contacts:givenName\" AS "        + TAG_FIRSTNAME          + ", " +
    "\"http://schemas.microsoft.com/exchange/keywords-utf8\" AS "
                                                    + TAG_CATEGORIES         + ", " +
    "\"urn:schemas:contacts:hobbies\" AS "          + TAG_HOBBY              + ", " +
    "\"urn:schemas:httpmail:importance\" AS "       + TAG_IMPORTANCE         + ", " +
    "\"urn:schemas:contacts:initials\" AS "         + TAG_INITIALS           + ", " +
    "\"urn:schemas:contacts:sn\" AS "               + TAG_LASTNAME           + ", " +
    "\"urn:schemas:contacts:language\" AS "         + TAG_LANGUAGE           + ", " +
    "\"urn:schemas:contacts:title\" AS "            + TAG_JOBTITLE           + ", " +
    "\"urn:schemas:contacts:middlename\" AS "       + TAG_MIDDLENAME         + ", " +
    "\"urn:schemas:contacts:mileage\" AS "          + TAG_MILEAGE            + ", " +
    "\"urn:schemas:contacts:nickname\" AS "         + TAG_NICKNAME           + ", " +
    "\"urn:schemas:contacts:o\" AS "                + TAG_ORGANIZATION       + ", " +
    "\"urn:schemas:contacts:sensitivity\" AS "      + TAG_SENSITIVITY        + ", " +
    "\"urn:schemas:contacts:spousecn\" AS "         + TAG_SPOUSECN           + ", " +
    "\"urn:schemas:contacts:namesuffix\" AS "       + TAG_SUFFIX             + ", " +
    "\"urn:schemas:contacts:personaltitle\" AS "    + TAG_TITLE              + ", " +
    "\"http://schemas.microsoft.com/exchange/yomiorganization\" AS "
                                                    + TAG_YOMI_COMPANY_NAME  + ", " +
    "\"http://schemas.microsoft.com/exchange/yomifirstname\" AS "
                                                    + TAG_YOMI_FIRST_NAME    + ", " +
    "\"http://schemas.microsoft.com/exchange/yomilastname\" AS "
                                                    + TAG_YOMI_LAST_NAME     + ", " +
    "\"urn:schemas:contacts:homeStreet\" AS "       + TAG_HOME_STREET        + ", " +
    "\"urn:schemas:contacts:homeCity\" AS "         + TAG_HOME_CITY          + ", " +
    "\"urn:schemas:contacts:homeState\" AS "        + TAG_HOME_STATE         + ", " +
    "\"urn:schemas:contacts:homePostalCode\" AS "   + TAG_HOME_POSTAL_CODE   + ", " +
    "\"urn:schemas:contacts:homepostofficebox\" AS "+
        TAG_HOME_POST_OFFICE_BOX                                             + ", " +
    "\"urn:schemas:contacts:homeCountry\" AS "      + TAG_HOME_COUNTRY       + ", " +

    "\"urn:schemas:contacts:street\" AS "           + TAG_WORK_STREET        + ", " +
    "\"urn:schemas:contacts:l\" AS "                + TAG_WORK_CITY          + ", " +
    "\"urn:schemas:contacts:st\" AS "               + TAG_WORK_STATE         + ", " +
    "\"urn:schemas:contacts:postalcode\" AS "       + TAG_WORK_POSTALCODE    + ", " +
    "\"urn:schemas:contacts:postofficebox\" AS "    +
        TAG_WORK_POST_OFFICE_BOX                                             + ", " +
    "\"urn:schemas:contacts:co\" AS "               + TAG_WORK_COUNTRY       + ", " +

    "\"urn:schemas:contacts:otherstreet\" AS "      + TAG_OTHER_STREET       + ", " +
    "\"urn:schemas:contacts:othercity\" AS "        + TAG_OTHER_CITY         + ", " +
    "\"urn:schemas:contacts:otherstate\" AS "       + TAG_OTHER_STATE        + ", " +
    "\"urn:schemas:contacts:otherpostalcode\" AS "  + TAG_OTHER_POSTALCODE   + ", " +
    "\"urn:schemas:contacts:otherpostofficebox\" AS "
        + TAG_OTHER_POST_OFFICE_BOX                                          + ", " +
    "\"urn:schemas:contacts:othercountry\" AS "     + TAG_OTHER_COUNTRY      + ", " +

    "\"urn:schemas:contacts:callbackphone\" AS "    + TAG_CALL_BACK_PHONE    + ", " +
    "\"urn:schemas:contacts:facsimiletelephonenumber\" "                     +
            "AS "                                   +
              TAG_FAC_SIMILE_TELEPHONE_NUMBER                                + ", " +
    "\"urn:schemas:contacts:homefax\" AS "          + TAG_HOME_FAX           + ", " +
    "\"urn:schemas:contacts:homePhone\" AS "        + TAG_HOME_PHONE         + ", " +
    "\"urn:schemas:contacts:homephone2\" AS "       + TAG_HOME_PHONE2        + ", " +
    "\"urn:schemas:contacts:mobile\" AS "           + TAG_MOBILE             + ", " +
    "\"urn:schemas:contacts:internationalisdnnumber\" AS " +
             TAG_ISDN_NUMBER                                                 + ", " +
    "\"urn:schemas:contacts:organizationmainphone\" "                        +
            "AS "                                   +
            TAG_ORGANIZATION_MAIN_PHONE                                      + ", " +
    "\"urn:schemas:contacts:otherfax\" AS "         + TAG_OTHER_FAX          + ", " +
    "\"urn:schemas:contacts:othermobile\" AS "      + TAG_OTHER_MOBILE       + ", " +
    "\"urn:schemas:contacts:otherTelephone\" AS "   + TAG_OTHER_TELEPHONE    + ", " +
    "\"urn:schemas:contacts:pager\" AS "            + TAG_PAGER              + ", " +
    "\"urn:schemas:contacts:secretaryphone\" AS "   + TAG_SECRETARY_PHONE    + ", " +
    "\"urn:schemas:contacts:telephoneNumber\" AS "  + TAG_TELEPHONE_NUMBER   + ", " +
    "\"urn:schemas:contacts:telephonenumber2\" AS " + TAG_TELEPHONE_NUMBER2  + ", " +
    "\"urn:schemas:contacts:telexnumber\" AS "      + TAG_TELEX_NUMBER       + ", " +

    "\"urn:schemas:contacts:businesshomepage\" AS " + TAG_BUSINESS_HOME_PAGE + ", " +
    "\"urn:schemas:contacts:personalHomePage\" AS " + TAG_PERSONAL_HOME_PAGE + ", " +

    "\"urn:schemas:contacts:department\" AS "       + TAG_DEPARTMENT         + ", " +
    "\"urn:schemas:contacts:roomnumber\" AS "       + TAG_OFFICE_LOCATION    + ", " +
    "\"urn:schemas:contacts:employeenumber\" AS "   +
        TAG_ORGANIZATION_ID_NUMBER                                           + ", " +
    "\"urn:schemas:contacts:profession\" AS "       + TAG_PROFESSION         + ", " +
    "\"urn:schemas:contacts:manager\" AS "          + TAG_MANAGER            + ", " +
    "\"urn:schemas:contacts:secretarycn\" AS "      + TAG_SECRETARY          + ", " +

    "\"urn:schemas:contacts:email1\" AS "           + TAG_EMAIL1             + ", " +
    "\"urn:schemas:contacts:email2\" AS "           + TAG_EMAIL2             + ", " +
    "\"urn:schemas:contacts:email3\" AS "           + TAG_EMAIL3             + ", " +
    "\"urn:schemas:contacts:email1addrtype\" AS "   +
        TAG_EMAIL1_ADDRESS_TYPE                                              + ", " +
    "\"urn:schemas:contacts:email2addrtype\" AS "   +
        TAG_EMAIL2_ADDRESS_TYPE                                              + ", " +
    "\"urn:schemas:contacts:email3addrtype\" AS "   +
        TAG_EMAIL3_ADDRESS_TYPE                                              + ", " +

    "\"urn:schemas:contacts:computernetworkname\" AS "
        + TAG_COMPUTER_NETWORK_NAME                                          + ", " +
    "\"DAV:displayname\", "                                                  +
    "\"DAV:getlastmodified\" AS getlastmodified, "                           +
    "\"DAV:isfolder\" AS isfolder "                                          +
    "FROM \"/{0}/{1}/{2}\"  "                                                +
    "{3} "                                                                   +
    "</D:sql>\n"                                                             +
    "</D:searchrequest>"                                                     ;

    private static final String
        FIELD_PREFIX                  = "urn:schemas:contacts:"    ;

    private static final String
        WEBDAV_INIT_EX                = "EX:"                      ;
    private static final String
        WEBDAV_INIT_HG                = "HG:"                      ;
    private static final String
        WEBDAV_INIT_HM                = "HM:"                      ;
    private static final String
        WEBDAV_INIT_HN                = "HN:"                      ;
    private static final String
        WEBDAV_INIT_HO                = "HO:"                      ;

    private static final String
        WEBDAV_INIT_MAPI              = "mapi:"                    ;

    private static final String
        WEBDAV_TAG_ANNIVERSARY        = WEBDAV_INIT_HM             +
                                        "weddinganniversary"       ;
    private static final String
        WEBDAV_TAG_ASSISTANT          = WEBDAV_INIT_HM             +
                                        "secretarycn"              ;
    private static final String
        WEBDAV_TAG_BIRTHDAY           = WEBDAV_INIT_HM             +
                                        "bday"                     ;
    private static final String
        WEBDAV_TAG_CATEGORIES         = WEBDAV_INIT_EX             +
                                        "keywords-utf8"            ;
    private static final String
        WEBDAV_TAG_CHILDREN           = WEBDAV_INIT_HM             +
                                        "childrensnames"           ;
    private static final String
        WEBDAV_TAG_CN                 = WEBDAV_INIT_HM             +
                                        "cn"                       ;
    private static final String
        WEBDAV_TAG_COMPANY            = WEBDAV_INIT_HM             +
                                        "o"                        ;
    private static final String
        WEBDAV_TAG_DEPARTMENT         = WEBDAV_INIT_HM             +
                                        "department"               ;
    private static final String
        WEBDAV_TAG_FILE_AS            = WEBDAV_INIT_HM             +
                                        "fileas"                   ;
    private static final String
        WEBDAV_TAG_FIRST_NAME         = WEBDAV_INIT_HM             +
                                        "givenName"                ;
    private static final String
        WEBDAV_TAG_HOBBY              = WEBDAV_INIT_HM             +
                                        "hobbies"                  ;
    private static final String
        WEBDAV_TAG_HOME_CITY          = WEBDAV_INIT_HM             +
                                        "homeCity"                 ;
    private static final String
        WEBDAV_TAG_HOME_COUNTRY       = WEBDAV_INIT_HM             +
                                        "homeCountry"              ;
    private static final String
        WEBDAV_TAG_HOME_POSTALCODE    = WEBDAV_INIT_HM             +
                                        "homePostalCode"           ;
    private static final String
        WEBDAVTAG_HOME_POST_OFFICE_BOX
                                      = WEBDAV_INIT_HM             +
                                        "homepostofficebox"        ;
    private static final String
        WEBDAV_TAG_HOME_STATE         = WEBDAV_INIT_HM             +
                                        "homeState"                ;
    private static final String
        WEBDAV_TAG_HOME_STREET        = WEBDAV_INIT_HM             +
                                        "homeStreet"               ;
    private static final String
        WEBDAV_TAG_IMPORTANCE         = WEBDAV_INIT_HN             +
                                        "importance"               ;
    private static final String
        WEBDAV_TAG_INITIALS           = WEBDAV_INIT_HM             +
                                        "initials"                 ;
    private static final String
        WEBDAV_TAG_JOB_TITLE          = WEBDAV_INIT_HM             +
                                        "title"                    ;
    private static final String
        WEBDAV_TAG_LANGUAGE          =  WEBDAV_INIT_HM             +
                                        "language";
    private static final String
        WEBDAV_TAG_LAST_NAME          = WEBDAV_INIT_HM             +
                                        "sn"                       ;
    private static final String
        WEBDAV_TAG_MANAGER            = WEBDAV_INIT_HM             +
                                        "manager"                  ;
    private static final String
        WEBDAV_TAG_MIDDLE_NAME        = WEBDAV_INIT_HM             +
                                        "middlename"               ;
    private static final String
        WEBDAV_TAG_MILEAGE            = WEBDAV_INIT_EX             +
                                        "mileage"                  ;
    private static final String
        WEBDAV_TAG_NICKNAME           = WEBDAV_INIT_HM             +
                                        "nickname"                 ;
    private static final String
        WEBDAV_TAG_OFFICE_LOCATION    = WEBDAV_INIT_HM             +
                                        "roomnumber"               ;
    private static final String
        WEBDAV_TAG_OTHER_CITY         = WEBDAV_INIT_HM             +
                                        "othercity"                ;
    private static final String
        WEBDAV_TAG_OTHER_COUNTRY      = WEBDAV_INIT_HM             +
                                        "othercountry"             ;
    private static final String
        WEBDAV_TAG_OTHER_POSTALCODE   = WEBDAV_INIT_HM             +
                                        "otherpostalcode"          ;
    private static final String
        WEBDAV_TAG_OTHER_POST_OFFICE_BOX
                                      = WEBDAV_INIT_HM             +
                                        "otherpostofficebox"       ;

    private static final String
        WEBDAV_TAG_OTHER_STATE        = WEBDAV_INIT_HM             +
                                        "otherstate"               ;
    private static final String
        WEBDAV_TAG_OTHER_STREET       = WEBDAV_INIT_HM             +
                                        "otherstreet"              ;
    private static final String
        WEBDAV_TAG_ORGANIZATION_ID_NUMBER
                                      = WEBDAV_INIT_HM             +
                                        "employeenumber"           ;
    private static final String
        WEBDAV_TAG_PROFESSION         = WEBDAV_INIT_HM             +
                                        "profession"               ;
    private static final String
        WEBDAV_TAG_SENSITIVITY        = WEBDAV_INIT_MAPI           +
                                        "sensitivity"              ;
    private static final String
        WEBDAV_TAG_SPOUSE             = WEBDAV_INIT_HM             +
                                        "spousecn"                 ;
    private static final String
        WEBDAV_TAG_SUBJECT1           = WEBDAV_INIT_HN             +
                                        "subject"                  ;
    private static final String
        WEBDAV_TAG_SUBJECT2           = WEBDAV_INIT_HO             +
                                        "subject"                  ;
    private static final String
        WEBDAV_TAG_SUFFIX             = WEBDAV_INIT_HM             +
                                        "namesuffix"               ;
    private static final String
        WEBDAV_TAG_TITLE              = WEBDAV_INIT_HM             +
                                        "personaltitle"            ;
    private static final String
        WEBDAV_TAG_WORK_CITY          = WEBDAV_INIT_HM             +
                                        "l"                        ;
    private static final String
        WEBDAV_TAG_WORK_COUNTRY       = WEBDAV_INIT_HM             +
                                        "co"                       ;
    private static final String
        WEBDAV_TAG_WORK_POSTALCODE    = WEBDAV_INIT_HM             +
                                        "postalcode"               ;
    private static final String
        WEBDAV_TAG_WORK_POST_OFFICE_BOX = WEBDAV_INIT_HM           +
                                        "postofficebox"            ;
    private static final String
        WEBDAV_TAG_WORK_STATE         = WEBDAV_INIT_HM             +
                                        "st"                       ;
    private static final String
        WEBDAV_TAG_WORK_STREET        = WEBDAV_INIT_HM             +
                                        "street"                   ;

    private static final String
        WEBDAV_TAG_YOMI_COMAPANY_NAME = WEBDAV_INIT_EX             +
                                        "yomiorganization"         ;
    private static final String
        WEBDAV_TAG_YOMI_FIRST_NAME    = WEBDAV_INIT_EX             +
                                        "yomifirstname"            ;
    private static final String
        WEBDAV_TAG_YOMI_LAST_NAME     = WEBDAV_INIT_EX             +
                                        "yomilastname"             ;
    private static final String
        WEBDAV_TAG_ASSISTANT_PHONE    = WEBDAV_INIT_HM             +
                                        "secretaryphone"           ;
    private static final String
        WEBDAV_TAG_CALLBACK_PHONE     = WEBDAV_INIT_HM             +
                                        "callbackphone"            ;
    private static final String
        WEBDAV_TAG_COMPANY_PHONE      = WEBDAV_INIT_HM             +
                                        "organizationmainphone"    ;
    private static final String
        WEBDAV_TAG_HOME_FAX           = WEBDAV_INIT_HM             +
                                        "homefax"                  ;
    private static final String
        WEBDAV_TAG_HOME_OTHER_PHONE   = WEBDAV_INIT_HM             +
                                        "homephone2"               ;
    private static final String
        WEBDAV_TAG_HOME_PHONE         = WEBDAV_INIT_HM             +
                                        "homePhone"                ;
    private static final String
        WEBDAV_TAG_MOBILE_OTHER_PHONE = WEBDAV_INIT_HM             +
                                        "othermobile"              ;
    private static final String
        WEBDAV_TAG_MOBILE_PHONE       = WEBDAV_INIT_HM             +
                                        "mobile"                   ;
    private static final String
        WEBDAV_TAG_OTHER_FAX          = WEBDAV_INIT_HM             +
                                        "otherfax"                 ;
    private static final String
        WEBDAV_TAG_OTHER_PHONE        = WEBDAV_INIT_HM             +
                                        "otherTelephone"           ;
    private static final String
        WEBDAV_TAG_PAGER              = WEBDAV_INIT_HM             +
                                        "pager"                    ;
    private static final String
        WEBDAV_TAG_TELEX_NUMBER       = WEBDAV_INIT_HM             +
                                        "telexnumber"              ;
    private static final String
        WEBDAV_TAG_WORK_FAX           = WEBDAV_INIT_HM             +
                                        "facsimiletelephonenumber" ;
    private static final String
        WEBDAV_TAG_WORK_OTHER_PHONE   = WEBDAV_INIT_HM             +
                                        "telephonenumber2"         ;
    private static final String
        WEBDAV_TAG_WORK_PHONE         = WEBDAV_INIT_HM             +
                                        "telephoneNumber"          ;

    private static final String
        WEBDAV_TAG_BUSINESS_HOME_PAGE = WEBDAV_INIT_HM             +
                                        "businesshomepage"         ;
    private static final String
        WEBDAV_TAG_PERSONAL_HOME_PAGE = WEBDAV_INIT_HM             +
                                        "personalHomePage"         ;
    private static final String
        WEBDAV_TAG_EMAIL1             = WEBDAV_INIT_HM             +
                                        "email1"                   ;
    private static final String
        WEBDAV_TAG_EMAIL2             = WEBDAV_INIT_HM             +
                                        "email2"                   ;
    private static final String
        WEBDAV_TAG_EMAIL3             = WEBDAV_INIT_HM             +
                                        "email3"                   ;
    private static final String
        WEBDAV_TAG_EMAIL1_ADDRESS_TYPE = WEBDAV_INIT_MAPI          +
                                        "email1addrtype"           ;
    private static final String
        WEBDAV_TAG_EMAIL2_ADDRESS_TYPE = WEBDAV_INIT_MAPI          +
                                        "email2addrtype"           ;
    private static final String
        WEBDAV_TAG_EMAIL3_ADDRESS_TYPE = WEBDAV_INIT_MAPI          +
                                        "email3addrtype"           ;

    private static final String
        WEBDAV_TAG_COMPUTER_NETWORK_NAME
                                      = WEBDAV_INIT_HM             +
                                        "computernetworkname"      ;

    private static final String
        FIELD_TYPE_ASSISTANT_NUMBER                     =
        "AssistantTelephoneNumber"                      ;
    private static final String
        FIELD_TYPE_BUSINESS_FAX_NUMBER                  =
        "BusinessFaxNumber"                             ;
    private static final String
        FIELD_TYPE_BUSINESS_OTHER_TELEPHONE_NUMBER      =
        "Business2TelephoneNumber"                      ;
    private static final String
        FIELD_TYPE_BUSINESS_TELEPHONE_NUMBER            =
        "BusinessTelephoneNumber"                       ;
    private static final String
        FIELD_TYPE_CALLBACK_NUMBER                      =
        "CallbackTelephoneNumber"                       ;
    private static final String
        FIELD_TYPE_CAR_TELEPHONE_NUMBER                 =
        "CarTelephoneNumber"                            ;
    private static final String
        FIELD_TYPE_COMPANY_MAIN_TELEPHONE_NUMBER        =
        "CompanyMainTelephoneNumber"                     ;
    public static final String
        FIELD_TYPE_EMAIL1_ADDRESS                       =
        "Email1Address"                                 ;
    public static final String
        FIELD_TYPE_EMAIL2_ADDRESS                       =
        "Email2Address"                                 ;
    private static final String
        FIELD_TYPE_EMAIL3_ADDRESS                       =
        "Email3Address"                                 ;
    private static final String
        FIELD_TYPE_JOB_TITLE                            =
        "JobTitle"                                      ;

    private static final String
        FIELD_TYPE_PERSONAL_HOME_PAGE                   =
        "HomeWebPage"                                   ;
    private static final String
        FIELD_TYPE_BUSINESS_HOME_PAGE                   =
        "WebPage"                                       ;
    private static final String
        FIELD_TYPE_HOME_TELEPHONE_NUMBER                =
        "HomeTelephoneNumber"                           ;
    private static final String
        FIELD_TYPE_HOME_OTHER_TELEPHONE_NUMBER          =
        "Home2TelephoneNumber"                          ;
    private static final String
        FIELD_TYPE_HOME_FAX_NUMBER                      =
        "HomeFaxNumber"                                 ;
    private static final String
        FIELD_TYPE_MOBILE_TELEPHONE_NUMBER              =
        "MobileTelephoneNumber"                         ;
    private static final String
        FIELD_TYPE_NOTE                                 =
        "Body"                                          ;
    private static final String
        FIELD_TYPE_OTHER_TELEPHONE_NUMBER               =
        "OtherTelephoneNumber"                          ;
    private static final String
        FIELD_TYPE_OTHER_FAX_NUMBER                     =
        "OtherFaxNumber"                                ;
    private static final String
        FIELD_TYPE_PAGER_NUMBER                         =
        "PagerNumber"                                   ;
    private static final String
        FIELD_TYPE_STATUS                               =
        "status"                                        ;

    //------------------------------------------------------------- Private data

    private WebDavHttpTransport  webDavHttp          = null ;

    //------------------------------------------------------------- Costructors

    public ContactDAO(String  exchangeServerHost ,
                      int     exchangeServerPort)
        throws DataAccessException {

        this.webDavHttp         = new WebDavHttpTransport(exchangeServerHost,
                                                          exchangeServerPort);
    }

    //------------------------------------------------------------- Public methods

    /**
     * insert / update contact<br>
     *
     * @param c
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new / updated contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact setContact(Contact c              ,
                              String  username       ,
                              String  credentials    ,
                              String  exchangeFolder )
    throws DataAccessException {

        PersonalDetail    personalDetail         = null ;
        BusinessDetail    businessDetail         = null ;
        Address           homeAddressBook        = null ;
        Address           workAddressBook        = null ;
        Address           otherAddressBook       = null ;

        Name              name                   = null ;
        Phone             phone                  = null;
        Email             email                  = null;
        WebPage           webPage                = null;

        List              webPages               = null ;
        List              emails                 = null ;
        List              webPagesPersonal       = null ;
        List              emailsBusiness         = null ;
        List              phonesPersonal         = null ;
        List              phonesBusiness         = null ;

        String            response               = null ;

        String            id                     = null ;

        StringBuffer      webDavContactMsg       = null ;
        String            webDavHeaderMsg        = null ;

        String            anniversary            = null ;
        String            birthday               = null ;
        String            categories             = null ;
        String            children               = null ;
        String            firstName              = null ;
        String            hobby                  = null ;
        Short             importance             = null ;
        String            initials               = null ;
        String            language               = null ;
        String            lastName               = null ;
        String            middleName             = null ;
        Integer           mileage                = null ;
        String            displayName            = null ;
        String            jobTitle               = null ;
        String            organization           = null ;
        Short             sensitivity            = null ;
        String            suffix                 = null ;
        String            title                  = null ;
        String            homeStreet             = null ;
        String            homeCity               = null ;
        String            homeState              = null ;
        String            homePostalCode         = null ;
        String            homePostOfficeBox      = null ;
        String            homeCountry            = null ;
        String            officeStreet           = null ;
        String            officeCity             = null ;
        String            officeState            = null ;
        String            officeCountry          = null ;
        String            officePostalCode       = null ;
        String            officePostOfficeBox    = null ;
        String            officeLocation         = null ;
        String            otherStreet            = null ;
        String            otherCity              = null ;
        String            otherState             = null ;
        String            otherPostalCode        = null ;
        String            otherPostOfficeBox     = null ;
        String            otherCountry           = null ;
        String            officePhone            = null ;
        String            homePhone              = null ;
        String            mobilePhone            = null ;
        String            assistantPhone         = null ;
        String            officeOtherPhone       = null ;
        String            officeFax              = null ;
        String            callBackPhone          = null ;
        String            mobileOtherPhone       = null ;
        String            organizationPhone      = null ;
        String            homeOtherPhone         = null ;
        String            homeFax                = null ;
        String            otherPhone             = null ;
        String            otherFax               = null ;
        String            pager                  = null ;
        String            personalHomePage       = null ;
        String            businessHomePage       = null ;
        String            department             = null ;
        String            profession             = null ;
        String            nickName               = null ;
        String            spouse                 = null ;
        String            manager                = null ;
        String            assistant              = null ;
        String            emailAddress1          = null ;
        String            emailAddress2          = null ;
        String            emailAddress3          = null ;

        String            phoneType              = null ;
        String            webPageType            = null ;

        String            value                  = null ;

        String            server                 = null ;
        String            resource               = null ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        XmlParser p = new XmlParser();

        personalDetail        = c.getPersonalDetail                 ()    ;
        businessDetail        = c.getBusinessDetail                 ()    ;
        name                  = c.getName                           ()    ;

        importance            = c.getImportance                     ()    ;
        sensitivity           = c.getSensitivity                    ()    ;
        mileage               = c.getMileage                        ()    ;
        language              = c.getLanguages                      ()    ;
        categories            = c.getCategories                     ()    ;

        if (personalDetail != null) {
            homeAddressBook       = personalDetail.getAddress       ()    ;
            otherAddressBook      = personalDetail.getOtherAddress  ()    ;
            webPages              = personalDetail.getWebPages      ()    ;
            emails                = personalDetail.getEmails        ()    ;
            phonesPersonal        = personalDetail.getPhones        ()    ;
            spouse                = personalDetail.getSpouse        ()    ;
            children              = personalDetail.getChildren      ()    ;
            birthday              = personalDetail.getBirthday      ()    ;
            anniversary           = personalDetail.getAnniversary   ()    ;
            hobby                 = personalDetail.getHobbies       ()    ;
            webPagesPersonal      = personalDetail.getWebPages      ()    ;
        }

        if (businessDetail != null) {
            workAddressBook       = businessDetail.getAddress       ()    ;
            phonesBusiness        = businessDetail.getPhones        ()    ;
            emailsBusiness        = businessDetail.getEmails        ()    ;
        }

        if (businessDetail != null) {
            assistant      = businessDetail.getAssistant            ()    ;
            manager        = businessDetail.getManager              ()    ;
            officeLocation = businessDetail.getOfficeLocation       ()    ;

            if (businessDetail.getCompany() != null) {
                organization     = businessDetail.getCompany    ().
                                            getPropertyValueAsString()    ;
            }
            if (businessDetail.getDepartment() != null) {
                department  = businessDetail.getDepartment ().
                                            getPropertyValueAsString()     ;
            }
            if (businessDetail.getRole() != null) {
                profession        = businessDetail.getRole       ().
                                            getPropertyValueAsString()     ;
            }
        }

        if (name != null) {
            if (name.getFirstName() != null) {
                firstName   = name.getFirstName        ().
                                        getPropertyValueAsString()         ;
            }
            if (name.getMiddleName() != null) {
                middleName  = name.getMiddleName       ().
                                        getPropertyValueAsString()         ;
            }
            if (name.getLastName() != null) {
                lastName    = name.getLastName         ().
                                        getPropertyValueAsString()         ;
            }
            if (name.getDisplayName() != null) {
                displayName = name.getDisplayName      ().
                                        getPropertyValueAsString()         ;
            }
            if (name.getNickname() != null) {
                nickName    = name.getNickname         ().
                                        getPropertyValueAsString()         ;
            }

            if (name.getInitials() != null) {
                initials    = name.getInitials         ().
                                        getPropertyValueAsString()         ;
            }

            if (name.getSuffix() != null) {
                suffix      = name.getSuffix           ().
                                        getPropertyValueAsString()         ;
            }
            if (name.getSalutation() != null) {
                title  = name.getSalutation       ().
                                        getPropertyValueAsString()         ;
            }

        }

        if (businessDetail.getTitles() !=null &&
            businessDetail.getTitles().size() > 0 &&
            ((Title) businessDetail.getTitles().get(0)) != null) {
            jobTitle = ((Title) businessDetail.getTitles().get(0)).
                                                getPropertyValueAsString();
        } else {
            jobTitle = null;
        }

        if (emails != null) {

            for (int i=0, l = emails.size(); i < l; i++) {

                email = (Email) emails.get(i);

                if ((FIELD_TYPE_EMAIL1_ADDRESS).equals(email.getEmailType())) {
                    emailAddress1 = email.getPropertyValueAsString();
                } else if ((FIELD_TYPE_EMAIL2_ADDRESS).
                            equals(email.getEmailType())) {
                    emailAddress2 = email.getPropertyValueAsString();
                } else if ((FIELD_TYPE_EMAIL3_ADDRESS).
                            equals(email.getEmailType())) {
                    emailAddress3 = email.getPropertyValueAsString();
                }

            }

        }

        //
        // phones
        //
        if (phonesPersonal != null) {

            for (int i=0, l = phonesPersonal.size(); i < l; i++) {

                phone = (Phone) phonesPersonal.get(i);

                phoneType = phone.getPhoneType();

                if ((FIELD_TYPE_HOME_TELEPHONE_NUMBER).
                                equals(phoneType))           {
                    homePhone = phone.getPropertyValueAsString();
                } if ((FIELD_TYPE_HOME_OTHER_TELEPHONE_NUMBER).
                                equals(phoneType))           {
                    homeOtherPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_HOME_FAX_NUMBER).
                                equals(phoneType))           {
                    homeFax = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_MOBILE_TELEPHONE_NUMBER).
                                equals(phoneType))           {
                    mobilePhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_CAR_TELEPHONE_NUMBER).
                                equals(phoneType))           {
                    mobileOtherPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_OTHER_TELEPHONE_NUMBER).
                                equals(phoneType))           {
                    otherPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_OTHER_FAX_NUMBER).
                                equals(phoneType))           {
                    otherFax = phone.getPropertyValueAsString();
                } else                                       {
                    //
                    // no save uknown property
                    //
                    continue;
                }

            }

        }

        //
        // phones business
        //
        if (phonesBusiness != null) {

            for (int i=0, l = phonesBusiness.size(); i < l; i++) {

                phone = (Phone) phonesBusiness.get(i);

                phoneType = phone.getPhoneType();

                if ((FIELD_TYPE_BUSINESS_TELEPHONE_NUMBER).
                            equals(phoneType))                    {
                    officePhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_BUSINESS_OTHER_TELEPHONE_NUMBER).
                            equals(phoneType))                    {
                    officeOtherPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_BUSINESS_FAX_NUMBER).
                            equals(phoneType))                    {
                    officeFax = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_COMPANY_MAIN_TELEPHONE_NUMBER).
                            equals(phoneType))                    {
                    organizationPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_PAGER_NUMBER).
                            equals(phoneType))                    {
                    pager = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_ASSISTANT_NUMBER).
                            equals(phoneType))                    {
                    assistantPhone = phone.getPropertyValueAsString();
                } else if ((FIELD_TYPE_CALLBACK_NUMBER).
                            equals(phoneType))                    {
                    callBackPhone = phone.getPropertyValueAsString();
                } else                                            {
                    //
                    // no save uknown property
                    //
                    continue;
                }

            }

        }

        //
        // webPages personal
        //

        if (webPagesPersonal != null) {

            for (int i=0, l = webPagesPersonal.size(); i < l; i++)     {

                webPage = (WebPage) webPagesPersonal.get(i);

                webPageType = webPage.getWebPageType();

                if ((FIELD_TYPE_PERSONAL_HOME_PAGE).equals(webPageType))         {
                    personalHomePage = webPage.getPropertyValueAsString();
                } else  if ((FIELD_TYPE_BUSINESS_HOME_PAGE).equals(webPageType)) {
                    businessHomePage = webPage.getPropertyValueAsString();
                 }                                                               {
                    //
                    // no save uknown property
                    //
                    continue;
                }

            }

        }

        if (homeAddressBook != null) {

            if (homeAddressBook.getStreet() != null) {
                homeStreet = homeAddressBook.getStreet().
                                            getPropertyValueAsString();
            }
            if (homeAddressBook.getCity() != null) {
                homeCity = homeAddressBook.getCity().
                                            getPropertyValueAsString();
            }
            if (homeAddressBook.getPostalCode() != null) {
                homePostalCode = homeAddressBook.getPostalCode().
                                            getPropertyValueAsString();
            }
            if (homeAddressBook.getPostOfficeAddress() != null) {
                homePostOfficeBox = homeAddressBook.
                                        getPostOfficeAddress().
                                                getPropertyValueAsString();
            }
            if (homeAddressBook.getState() != null) {
                homeState = homeAddressBook.getState().
                                            getPropertyValueAsString();
            }
            if (homeAddressBook.getCountry()!= null) {
                homeCountry = homeAddressBook.getCountry().
                                            getPropertyValueAsString();
            }

        }

        if (otherAddressBook != null) {

            if (otherAddressBook.getStreet() != null) {
                otherStreet = otherAddressBook.getStreet().
                                        getPropertyValueAsString();
            }
            if (otherAddressBook.getCity() != null) {
                otherCity = otherAddressBook.getCity().
                                        getPropertyValueAsString();
            }
            if (otherAddressBook.getPostalCode() != null) {
                otherPostalCode = otherAddressBook.getPostalCode().
                                        getPropertyValueAsString();
            }
            if (otherAddressBook.getPostOfficeAddress() != null) {
                otherPostOfficeBox = otherAddressBook.
                                        getPostOfficeAddress().
                                                getPropertyValueAsString();
            }
            if (otherAddressBook.getState() != null) {
                otherState = otherAddressBook.getState().
                                        getPropertyValueAsString();
            }
            if (homeAddressBook.getCountry()!= null) {
                otherCountry = otherAddressBook.getCountry().
                                        getPropertyValueAsString();
            }

        }

        if (workAddressBook != null) {

            if (workAddressBook.getStreet() != null) {
                officeStreet = workAddressBook.getStreet().
                                    getPropertyValueAsString();
            }
            if (workAddressBook.getCity() != null) {
                officeCity = workAddressBook.getCity().
                                    getPropertyValueAsString();
            }
            if (workAddressBook.getPostalCode() != null) {
                officePostalCode = workAddressBook.getPostalCode().
                                    getPropertyValueAsString();
            }
            if (workAddressBook.getPostOfficeAddress() != null) {
                officePostOfficeBox = workAddressBook.
                                        getPostOfficeAddress().
                                                getPropertyValueAsString();
            }
            if (workAddressBook.getState() != null) {
                officeState = workAddressBook.getState().
                                    getPropertyValueAsString();
            }
            if (workAddressBook.getCountry()!= null) {
                officeCountry = workAddressBook.getCountry().
                                    getPropertyValueAsString();
            }

        }

        webDavContactMsg = new StringBuffer();

        webDavContactMsg.
            append
                ("<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                   +
                 "<D:propertyupdate xmlns:D=\"DAV:\" "                             +
                 "xmlns:EX=\"http://schemas.microsoft.com/exchange/\" "            +
                 "xmlns:HM=\"urn:schemas:contacts:\" "                             +
                 "xmlns:HG=\"urn:schemas:calendar:\" "                             +
                 "xmlns:HN=\"urn:schemas:httpmail:\" "                             +
                 "xmlns:x=\"xml:\" "                                               +
                 "xmlns:mapi=\"http://schemas.microsoft.com/mapi/\" "              +
                 "xmlns:HO=\"urn:schemas:mailheader:\">\n"                         +
                 "<D:set>\n"                                                       +
                 "<D:prop>\n"                                                      +
                 "<D:contentclass>urn:content-classes:person</D:contentclass>\n"   +
                 "<EX:outlookmessageclass>IPM.Contact</EX:outlookmessageclass>\n") ;


        firstName   = StringTools.escapeXml(firstName  );
        lastName    = StringTools.escapeXml(lastName   );
        middleName  = StringTools.escapeXml(middleName );
        initials    = StringTools.escapeXml(initials   );

        //
        // full name variables
        //
        String fnTitle      = title      ;
        String fnFirstName  = firstName  ;
        String fnMiddleName = middleName ;
        String fnLastName   = lastName   ;
        String fnSuffix     = suffix     ;

        if ((title      != null && title.length      () > 0 ) &&
            ((firstName  != null && firstName.length  () > 0 ) ||
            (middleName != null && middleName.length () > 0 )  ||
            (lastName   != null && lastName.length   () > 0 )  ||
            (suffix     != null && suffix.length     () > 0 ))) {

            fnTitle = fnTitle + " ";

        }

        if ((firstName  != null && firstName.length  () > 0 ) &&
            ((middleName != null && middleName.length () > 0 ) ||
            (lastName   != null && lastName.length   () > 0 )  ||
            (suffix     != null && suffix.length     () > 0 ))) {

            fnFirstName = fnFirstName + " ";

        }

        if ((middleName != null && middleName.length () > 0 ) &&
            ((lastName   != null && lastName.length   () > 0 ) ||
            (suffix     != null && suffix.length     () > 0 ))) {

            fnMiddleName = fnMiddleName + " ";

        }

        if ((lastName != null && lastName.length () > 0) &&
            (suffix   != null && suffix.length   () > 0)) {

            fnLastName = fnLastName + " ";

        }

        value = "";

        if (fnTitle != null) {
            value = fnTitle;
        }

        if (fnFirstName != null) {
            value = value + fnFirstName;
        }

        if (fnMiddleName != null) {
            value = value + fnMiddleName;
        }

        if (fnLastName != null) {
            value = value + fnLastName;
        }

        if (fnSuffix != null) {
            value = value + fnSuffix;
        }

        if (value != null && value.length() > 0) {
            webDavContactMsg.append("<"           +
                                    WEBDAV_TAG_CN +
                                    ">"           +
                                    value         +
                                    "</"          +
                                    WEBDAV_TAG_CN +
                                    ">\n"         );
        }

        value = null;
        value = (lastName != null ? lastName : "")                        +
                (((firstName != null && firstName.length() > 0)
                    && (lastName != null && lastName.length() > 0))
                    ? ", " : "")                                          +
                (firstName != null ? firstName : "")                      +
                (((middleName != null && middleName.length() > 0)
                    && (firstName !=null && firstName.length() > 0))
                    ? " "  : "")                                          +
                (middleName != null ? middleName : "")                    ;

        if (value != null) {
            webDavContactMsg.append("<"                +
                                    WEBDAV_TAG_FILE_AS +
                                    ">"                +
                                    value              +
                                    "</"               +
                                    WEBDAV_TAG_FILE_AS +
                                    ">\n"              );
        }

        value = null;
        value = (lastName != null ? lastName : "")                        +
                (((firstName != null && firstName.length() > 0)
                    && (lastName != null && lastName.length() > 0))
                    ? " " : "")                                           +
                (firstName != null ? firstName : "")                      ;

        if (value != null) {
            webDavContactMsg.append("<"                 +
                                    WEBDAV_TAG_SUBJECT1 +
                                    ">"                 +
                                    value               +
                                    "</"                +
                                    WEBDAV_TAG_SUBJECT1 +
                                    ">\n"               );
        }

        value = null;
        value = (lastName != null ? lastName : "")                        +
                (((firstName != null && firstName.length() > 0)
                    && (lastName != null && lastName.length() > 0 ))
                    ? " " : "")                                           +
                (firstName != null ? firstName : "")                      ;                                           ;

        if (value != null) {
            webDavContactMsg.append("<"                 +
                                    WEBDAV_TAG_SUBJECT2 +
                                    ">"                 +
                                    value               +
                                    "</"                +
                                    WEBDAV_TAG_SUBJECT2 +
                                    ">\n"               );
        }

        if (firstName != null) {
            webDavContactMsg.append("<"                   +
                                    WEBDAV_TAG_FIRST_NAME +
                                    ">"                   +
                                    firstName             +
                                    "</"                  +
                                    WEBDAV_TAG_FIRST_NAME +
                                    ">\n"                 );
        }

        if (lastName != null) {
            webDavContactMsg.append("<"                  +
                                    WEBDAV_TAG_LAST_NAME +
                                    ">"                  +
                                    lastName             +
                                    "</"                 +
                                    WEBDAV_TAG_LAST_NAME +
                                    ">\n"                );
        }

        if (middleName != null) {
            webDavContactMsg.append("<"                    +
                                    WEBDAV_TAG_MIDDLE_NAME +
                                    ">"                    +
                                    middleName             +
                                    "</"                   +
                                    WEBDAV_TAG_MIDDLE_NAME +
                                    ">\n"                  );
        }

        if (language != null) {
            language  = StringTools.escapeXml(language);
            webDavContactMsg.append("<"                 +
                                    WEBDAV_TAG_LANGUAGE +
                                    ">"                 +
                                    language            +
                                    "</"                +
                                    WEBDAV_TAG_LANGUAGE +
                                    ">\n"               );
        }

        if (suffix != null) {
            suffix  = StringTools.escapeXml(suffix);
            webDavContactMsg.append("<"               +
                                    WEBDAV_TAG_SUFFIX +
                                    ">"               +
                                    suffix            +
                                    "</"              +
                                    WEBDAV_TAG_SUFFIX +
                                    ">\n"             );
        }

        if (title != null) {
            title  = StringTools.escapeXml(title);
            webDavContactMsg.append("<"              +
                                    WEBDAV_TAG_TITLE +
                                    ">"              +
                                    title            +
                                    "</"             +
                                    WEBDAV_TAG_TITLE +
                                    ">\n"            );
        }

        if (initials != null) {
                    webDavContactMsg.append("<"                 +
                                            WEBDAV_TAG_INITIALS +
                                            ">"                 +
                                            initials            +
                                            "</"                +
                                            WEBDAV_TAG_INITIALS +
                                            ">\n"               );
        }

        if (importance != null) {
                    webDavContactMsg.append("<"                    +
                                        WEBDAV_TAG_IMPORTANCE      +
                                        ">"                        +
                                        String.valueOf(importance) +
                                        "</"                       +
                                        WEBDAV_TAG_IMPORTANCE      +
                                        ">\n"                      );
        }

        if (sensitivity != null) {
                    webDavContactMsg.append("<"                     +
                                        WEBDAV_TAG_SENSITIVITY      +
                                        ">"                         +
                                        String.valueOf(sensitivity) +
                                        "</"                        +
                                        WEBDAV_TAG_SENSITIVITY      +
                                        ">\n"                       );
        }

        if (mileage != null) {
                    webDavContactMsg.append("<"                 +
                                        WEBDAV_TAG_MILEAGE      +
                                        ">"                     +
                                        String.valueOf(mileage) +
                                        "</"                    +
                                        WEBDAV_TAG_MILEAGE      +
                                        ">\n"                   );
        }


        try {

            if (anniversary != null) {
                webDavContactMsg.append("<"                     +
                                        WEBDAV_TAG_ANNIVERSARY  +
                                        ">"                     +
                                        XmlParser.
                                            PDIToWebDavAnniversary
                                                (anniversary)   +
                                        "</"                    +
                                        WEBDAV_TAG_ANNIVERSARY  +
                                        ">\n"                   );
            }

            if (birthday != null) {
                webDavContactMsg.append("<"                     +
                                        WEBDAV_TAG_BIRTHDAY     +
                                        ">"                     +
                                        XmlParser.
                                            PDIToWebDavAnniversary
                                                (birthday)      +
                                        "</"                    +
                                        WEBDAV_TAG_BIRTHDAY     +
                                        ">\n"                   );
            }

        } catch (Exception e) {
            throw new DataAccessException(e.getMessage());
        }

        if (categories != null) {

            categories  = StringTools.escapeXml(categories);
            webDavContactMsg.append("<"                     +
                                    WEBDAV_TAG_CATEGORIES   +
                                    ">"                     +
                                    XmlParser.
                                        fieldToMultipleTag
                                            (categories)    +
                                    "</"                    +
                                    WEBDAV_TAG_CATEGORIES   +
                                    ">\n"                   );
        }

        if (children != null) {
            children  = StringTools.escapeXml(children);
            webDavContactMsg.append("<"                     +
                                    WEBDAV_TAG_CHILDREN     +
                                    ">"                     +
                                    XmlParser.
                                        fieldToMultipleTag
                                            (children)      +
                                    "</"                    +
                                    WEBDAV_TAG_CHILDREN     +
                                    ">\n"                   );
        }

        if (hobby != null) {
            hobby  = StringTools.escapeXml(hobby);
            webDavContactMsg.append("<"                     +
                                    WEBDAV_TAG_HOBBY        +
                                    ">"                     +
                                    hobby                   +
                                    "</"                    +
                                    WEBDAV_TAG_HOBBY        +
                                    ">\n"                   );
        }

        if (jobTitle != null) {
            jobTitle  = StringTools.escapeXml(jobTitle);
            webDavContactMsg.append("<"                  +
                                    WEBDAV_TAG_JOB_TITLE +
                                    ">"                  +
                                    jobTitle             +
                                    "</"                 +
                                    WEBDAV_TAG_JOB_TITLE +
                                    ">\n"                );
        }

        if (officeLocation != null) {
            officeLocation  = StringTools.escapeXml(officeLocation);
            webDavContactMsg.append("<"                        +
                                    WEBDAV_TAG_OFFICE_LOCATION +
                                    ">"                        +
                                    officeLocation             +
                                    "</"                       +
                                    WEBDAV_TAG_OFFICE_LOCATION +
                                    ">\n"                      );
        }

        if (organization != null) {
            organization  = StringTools.escapeXml(organization);
            webDavContactMsg.append("<"                +
                                    WEBDAV_TAG_COMPANY +
                                    ">"                +
                                    organization       +
                                    "</"               +
                                    WEBDAV_TAG_COMPANY +
                                    ">\n"              );
        }

        if (homeStreet != null) {
            homeStreet  = StringTools.escapeXml(homeStreet);
            webDavContactMsg.append("<"                    +
                                    WEBDAV_TAG_HOME_STREET +
                                    ">"                    +
                                    homeStreet             +
                                    "</"                   +
                                    WEBDAV_TAG_HOME_STREET +
                                    ">\n"                  );
        }

        if (homeCity != null) {
            homeCity  = StringTools.escapeXml(homeCity);
            webDavContactMsg.append("<"                  +
                                    WEBDAV_TAG_HOME_CITY +
                                    ">"                  +
                                    homeCity             +
                                    "</"                 +
                                    WEBDAV_TAG_HOME_CITY +
                                    ">\n"                );
        }

        if (homeState != null) {
            homeState  = StringTools.escapeXml(homeState);
            webDavContactMsg.append("<"                   +
                                    WEBDAV_TAG_HOME_STATE +
                                    ">"                   +
                                    homeState             +
                                    "</"                  +
                                    WEBDAV_TAG_HOME_STATE +
                                    ">\n"                 );
        }

        if (homePostalCode != null) {
            homePostalCode  = StringTools.escapeXml(homePostalCode);
            webDavContactMsg.append("<"                        +
                                    WEBDAV_TAG_HOME_POSTALCODE +
                                    ">"                        +
                                    homePostalCode             +
                                    "</"                       +
                                    WEBDAV_TAG_HOME_POSTALCODE +
                                    ">\n"                      );
        }

        if (homePostOfficeBox != null) {
            homePostOfficeBox  = StringTools.escapeXml(homePostOfficeBox);
            webDavContactMsg.append("<"                              +
                                    WEBDAVTAG_HOME_POST_OFFICE_BOX   +
                                    ">"                              +
                                    homePostOfficeBox                +
                                    "</"                             +
                                    WEBDAVTAG_HOME_POST_OFFICE_BOX   +
                                    ">\n"                            );
        }

        if (homeCountry != null) {
            homeCountry  = StringTools.escapeXml(homeCountry);
            webDavContactMsg.append("<"                     +
                                    WEBDAV_TAG_HOME_COUNTRY +
                                    ">"                     +
                                    homeCountry             +
                                    "</"                    +
                                    WEBDAV_TAG_HOME_COUNTRY +
                                    ">\n"                   );
        }

        if (officeStreet != null) {
            officeStreet  = StringTools.escapeXml(officeStreet);
            webDavContactMsg.append("<"                    +
                                    WEBDAV_TAG_WORK_STREET +
                                    ">"                    +
                                    officeStreet           +
                                    "</"                   +
                                    WEBDAV_TAG_WORK_STREET +
                                    ">\n"                  );
        }

        if (officeCity != null) {
            officeCity  = StringTools.escapeXml(officeCity);
            webDavContactMsg.append("<" +
                                    WEBDAV_TAG_WORK_CITY +
                                    ">"                  +
                                    officeCity           +
                                    "</"                 +
                                    WEBDAV_TAG_WORK_CITY +
                                    ">\n"                );
        }

        if (officeState != null) {
            officeState  = StringTools.escapeXml(officeState);
            webDavContactMsg.append("<"                   +
                                    WEBDAV_TAG_WORK_STATE +
                                    ">"                   +
                                    officeState           +
                                    "</"                  +
                                    WEBDAV_TAG_WORK_STATE +
                                    ">\n"                 );
        }

        if (officePostalCode != null) {
            officePostalCode  = StringTools.escapeXml(officePostalCode);
            webDavContactMsg.append("<"                        +
                                    WEBDAV_TAG_WORK_POSTALCODE +
                                    ">"                        +
                                    officePostalCode           +
                                    "</"                       +
                                    WEBDAV_TAG_WORK_POSTALCODE +
                                    ">\n"                      );
        }

        if (officePostOfficeBox != null) {
            officePostOfficeBox  = StringTools.escapeXml(officePostOfficeBox);
            webDavContactMsg.append("<"                             +
                                    WEBDAV_TAG_WORK_POST_OFFICE_BOX +
                                    ">"                             +
                                    officePostOfficeBox             +
                                    "</"                            +
                                    WEBDAV_TAG_WORK_POST_OFFICE_BOX +
                                    ">\n"                      );
        }

        if (officeCountry != null) {
            officeCountry  = StringTools.escapeXml(officeCountry);
            webDavContactMsg.append("<"                     +
                                    WEBDAV_TAG_WORK_COUNTRY +
                                    ">"                     +
                                    officeCountry           +
                                    "</"                    +
                                    WEBDAV_TAG_WORK_COUNTRY +
                                    ">\n"                   );
        }

        if (otherStreet != null) {
            otherStreet  = StringTools.escapeXml(otherStreet);
            webDavContactMsg.append("<"                     +
                                    WEBDAV_TAG_OTHER_STREET +
                                    ">"                     +
                                    otherStreet             +
                                    "</"                    +
                                    WEBDAV_TAG_OTHER_STREET +
                                    ">\n"                   );
        }

        if (otherCity != null) {
            otherCity  = StringTools.escapeXml(otherCity);
            webDavContactMsg.append("<"                   +
                                    WEBDAV_TAG_OTHER_CITY +
                                    ">"                   +
                                    otherCity             +
                                    "</"                  +
                                    WEBDAV_TAG_OTHER_CITY +
                                    ">\n"                 );
        }

        if (otherState != null) {
            otherState  = StringTools.escapeXml(otherState);
            webDavContactMsg.append("<"                    +
                                    WEBDAV_TAG_OTHER_STATE +
                                    ">"                    +
                                    otherState             +
                                    "</"                   +
                                    WEBDAV_TAG_OTHER_STATE +
                                    ">\n"                  );
        }

        if (otherPostalCode != null) {
            otherPostalCode  = StringTools.escapeXml(otherPostalCode);
            webDavContactMsg.append("<"                         +
                                    WEBDAV_TAG_OTHER_POSTALCODE +
                                    ">"                         +
                                    otherPostalCode             +
                                    "</"                        +
                                    WEBDAV_TAG_OTHER_POSTALCODE +
                                    ">\n"                       );
        }

        if (otherPostOfficeBox != null) {
            otherPostOfficeBox  = StringTools.escapeXml(otherPostOfficeBox);
            webDavContactMsg.append("<"                         +
                                    WEBDAV_TAG_OTHER_POST_OFFICE_BOX +
                                    ">"                         +
                                    otherPostOfficeBox          +
                                    "</"                        +
                                    WEBDAV_TAG_OTHER_POST_OFFICE_BOX +
                                    ">\n"                       );
        }


        if (otherCountry != null) {
            otherCountry  = StringTools.escapeXml(otherCountry);
            webDavContactMsg.append("<"                      +
                                    WEBDAV_TAG_OTHER_COUNTRY +
                                    ">"                      +
                                    otherCountry             +
                                    "</"                     +
                                    WEBDAV_TAG_OTHER_COUNTRY +
                                    ">\n"                    );
        }

        if (officePhone != null) {
            officePhone  = StringTools.escapeXml(officePhone);
            webDavContactMsg.append("<"                   +
                                    WEBDAV_TAG_WORK_PHONE +
                                    ">"                   +
                                    officePhone           +
                                    "</"                  +
                                    WEBDAV_TAG_WORK_PHONE +
                                    ">\n"                 );
        }

        if (homePhone != null) {
            homePhone  = StringTools.escapeXml(homePhone);
            webDavContactMsg.append("<"                   +
                                    WEBDAV_TAG_HOME_PHONE +
                                    ">"                   +
                                    homePhone             +
                                    "</"                  +
                                    WEBDAV_TAG_HOME_PHONE +
                                    ">\n"                 );
        }

        if (mobilePhone != null) {
            mobilePhone  = StringTools.escapeXml(mobilePhone);
            webDavContactMsg.append("<"                     +
                                    WEBDAV_TAG_MOBILE_PHONE +
                                    ">"                     +
                                    mobilePhone             +
                                    "</"                    +
                                    WEBDAV_TAG_MOBILE_PHONE +
                                    ">\n"                   );
        }

        if (assistantPhone != null) {
            assistantPhone  = StringTools.escapeXml(assistantPhone);
            webDavContactMsg.append("<" +
                                    WEBDAV_TAG_ASSISTANT_PHONE +
                                    ">"                        +
                                    assistantPhone             +
                                    "</"                       +
                                    WEBDAV_TAG_ASSISTANT_PHONE +
                                    ">\n"                      );
        }

        if (officeOtherPhone != null) {
            officeOtherPhone  = StringTools.escapeXml(officeOtherPhone);
            webDavContactMsg.append("<"                         +
                                    WEBDAV_TAG_WORK_OTHER_PHONE +
                                    ">"                         +
                                    officeOtherPhone            +
                                    "</"                        +
                                    WEBDAV_TAG_WORK_OTHER_PHONE +
                                    ">\n"                       );
        }

        if (officeFax != null) {
            officeFax  = StringTools.escapeXml(officeFax);
            webDavContactMsg.append("<"                 +
                                    WEBDAV_TAG_WORK_FAX +
                                    ">"                 +
                                    officeFax           +
                                    "</"                +
                                    WEBDAV_TAG_WORK_FAX +
                                    ">\n"               );
        }

        if (callBackPhone != null) {
            callBackPhone  = StringTools.escapeXml(callBackPhone);
            webDavContactMsg.append("<"                             +
                                    WEBDAV_TAG_CALLBACK_PHONE       +
                                    ">"                             +
                                    callBackPhone                   +
                                    "</"                            +
                                    WEBDAV_TAG_CALLBACK_PHONE       +
                                    ">\n"                           );
        }

        if (mobileOtherPhone != null) {
            mobileOtherPhone  = StringTools.escapeXml(mobileOtherPhone);
            webDavContactMsg.append("<"                           +
                                    WEBDAV_TAG_MOBILE_OTHER_PHONE +
                                    ">"                           +
                                    mobileOtherPhone              +
                                    "</"                          +
                                    WEBDAV_TAG_MOBILE_OTHER_PHONE +
                                    ">\n"                         );
        }


        if (organizationPhone != null) {
            organizationPhone  = StringTools.escapeXml(organizationPhone);
            webDavContactMsg.append("<"                      +
                                    WEBDAV_TAG_COMPANY_PHONE +
                                    ">"                      +
                                    organizationPhone        +
                                    "</"                     +
                                    WEBDAV_TAG_COMPANY_PHONE +
                                    ">\n"                    );
        }

        if (homeOtherPhone != null) {
            homeOtherPhone  = StringTools.escapeXml(homeOtherPhone);
            webDavContactMsg.append("<"                         +
                                    WEBDAV_TAG_HOME_OTHER_PHONE +
                                    ">"                         +
                                    homeOtherPhone              +
                                    "</"                        +
                                    WEBDAV_TAG_HOME_OTHER_PHONE +
                                    ">\n"                       );
        }

        if (homeFax != null) {
            homeFax  = StringTools.escapeXml(homeFax);
            webDavContactMsg.append("<"                 +
                                    WEBDAV_TAG_HOME_FAX +
                                    ">"                 +
                                    homeFax             +
                                    "</"                +
                                    WEBDAV_TAG_HOME_FAX +
                                    ">\n"               );
        }

        if (otherPhone != null) {
            otherPhone  = StringTools.escapeXml(otherPhone);
            webDavContactMsg.append("<"                    +
                                    WEBDAV_TAG_OTHER_PHONE +
                                    ">"                    +
                                    otherPhone             +
                                    "</"                   +
                                    WEBDAV_TAG_OTHER_PHONE +
                                    ">\n"                  );
        }

        if (otherFax != null) {
            otherFax  = StringTools.escapeXml(otherFax);
            webDavContactMsg.append("<"                  +
                                    WEBDAV_TAG_OTHER_FAX +
                                    ">"                  +
                                    otherFax             +
                                    "</"                 +
                                    WEBDAV_TAG_OTHER_FAX +
                                    ">\n"                );
        }

        if (pager != null) {
            pager  = StringTools.escapeXml(pager);
            webDavContactMsg.append("<"              +
                                    WEBDAV_TAG_PAGER +
                                    ">"              +
                                    pager            +
                                    "</"             +
                                    WEBDAV_TAG_PAGER +
                                    ">\n"            );
        }

        if (department != null) {
            department  = StringTools.escapeXml(department);
            webDavContactMsg.append("<"                   +
                                    WEBDAV_TAG_DEPARTMENT +
                                    ">"                   +
                                    department            +
                                    "</"                  +
                                    WEBDAV_TAG_DEPARTMENT +
                                    ">\n"                 );
        }

        if (profession != null) {
            profession  = StringTools.escapeXml(profession);
            webDavContactMsg.append("<"                   +
                                    WEBDAV_TAG_PROFESSION +
                                    ">"                   +
                                    profession            +
                                    "</"                  +
                                    WEBDAV_TAG_PROFESSION +
                                    ">\n"                 );
        }

        if (nickName != null) {
            nickName  = StringTools.escapeXml(nickName);
            webDavContactMsg.append("<"                 +
                                    WEBDAV_TAG_NICKNAME +
                                    ">"                 +
                                    nickName            +
                                    "</"                +
                                    WEBDAV_TAG_NICKNAME +
                                    ">\n"               );
        }

        if (spouse != null) {
            spouse  = StringTools.escapeXml(spouse);
            webDavContactMsg.append("<"               +
                                    WEBDAV_TAG_SPOUSE +
                                    ">"               +
                                    spouse            +
                                    "</"              +
                                    WEBDAV_TAG_SPOUSE +
                                    ">\n"             );
        }

        if (manager != null) {
            manager  = StringTools.escapeXml(manager);
            webDavContactMsg.append("<"                +
                                    WEBDAV_TAG_MANAGER +
                                    ">"                +
                                    manager            +
                                    "</"               +
                                    WEBDAV_TAG_MANAGER +
                                    ">\n"              );
        }

        if (assistant != null) {
            assistant  = StringTools.escapeXml(assistant);
            webDavContactMsg.append("<"                  +
                                    WEBDAV_TAG_ASSISTANT +
                                    ">"                  +
                                    assistant            +
                                    "</"                 +
                                    WEBDAV_TAG_ASSISTANT +
                                    ">\n"                );
        }

        if (personalHomePage != null) {
            personalHomePage  = StringTools.escapeXml(personalHomePage);
            webDavContactMsg.append("<"                            +
                                    WEBDAV_TAG_PERSONAL_HOME_PAGE  +
                                    ">"                            +
                                    personalHomePage               +
                                    "</"                           +
                                    WEBDAV_TAG_PERSONAL_HOME_PAGE  +
                                    ">\n"                          );
        }

        if (businessHomePage != null) {
            businessHomePage  = StringTools.escapeXml(businessHomePage);
            webDavContactMsg.append("<"                           +
                                    WEBDAV_TAG_BUSINESS_HOME_PAGE +
                                    ">"                           +
                                    businessHomePage              +
                                    "</"                          +
                                    WEBDAV_TAG_BUSINESS_HOME_PAGE +
                                    ">\n"                         );
        }

        if (emailAddress1 != null) {
            emailAddress1  = StringTools.escapeXml(emailAddress1);
            webDavContactMsg.append("<"               +
                                    WEBDAV_TAG_EMAIL1 +
                                    ">"               +
                                    emailAddress1     +
                                    "</"              +
                                    WEBDAV_TAG_EMAIL1 +
                                    ">\n"             );

            webDavContactMsg.append("<"                              +
                                    WEBDAV_TAG_EMAIL1_ADDRESS_TYPE   +
                                    ">"                              +
                                    FIELD_TYPE_EMAIL1_ADDRESS        +
                                    "</"                             +
                                    WEBDAV_TAG_EMAIL1_ADDRESS_TYPE   +
                                    ">\n"                            );
        }

        if (emailAddress2 != null) {
            emailAddress2  = StringTools.escapeXml(emailAddress2);
            webDavContactMsg.append("<"               +
                                    WEBDAV_TAG_EMAIL2 +
                                    ">"               +
                                    emailAddress2     +
                                    "</"              +
                                    WEBDAV_TAG_EMAIL2 +
                                    ">\n"             );

            webDavContactMsg.append("<"               +
                                    WEBDAV_TAG_EMAIL2_ADDRESS_TYPE   +
                                    ">"                              +
                                    FIELD_TYPE_EMAIL2_ADDRESS        +
                                    "</"                             +
                                    WEBDAV_TAG_EMAIL2_ADDRESS_TYPE   +
                                    ">\n"                            );

        }

        if (emailAddress3 != null) {
            emailAddress3  = StringTools.escapeXml(emailAddress3);
            webDavContactMsg.append("<"               +
                                    WEBDAV_TAG_EMAIL3 +
                                    ">"               +
                                    emailAddress3     +
                                    "</"              +
                                    WEBDAV_TAG_EMAIL3 +
                                    ">\n"             );

            webDavContactMsg.append("<"                              +
                                    WEBDAV_TAG_EMAIL3_ADDRESS_TYPE   +
                                    ">"                              +
                                    FIELD_TYPE_EMAIL3_ADDRESS        +
                                    "</"                             +
                                    WEBDAV_TAG_EMAIL3_ADDRESS_TYPE   +
                                    ">\n"                            );
        }

        webDavContactMsg.append("</D:prop>\n"         +
                                "</D:set>\n"          +
                                "</D:propertyupdate>" );                                                  ;


        webDavHeaderMsg
            = MessageFormat.format(ItemDAO.WEBDAV_HEADER_PROPPATCH,
                new Object[] {
                        "/"                   +
                        server                +
                        "/"                   +
                        username              +
                        "/"                   +
                        resource              ,
                        c.getHref()
              }
        );


        try {
             response = this.webDavHttp.sendRequest(webDavHeaderMsg             ,
                                                    credentials                 ,
                                                    webDavContactMsg.toString() ,
                                                    FILE_ENCODING               );
        } catch (IOException e) {
             throw new DataAccessException("Error getting Exchange server response",
                                            e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                       +
                                          username                      +
                                          " URI "                       +
                                          exchangeFolder                +
                                          " setting exchange contact"   ,
                                          e)                            ;

        }

        try {
            id = XmlParser.getRuidFromResponse(response);
        } catch (XmlParseException e) {
            throw new DataAccessException("Error getting resource id", e);
        }

        c.setId(id);

        return c;

    }

    /**
     * Delete a <i>Contact</i> from Exchange server.
     *
     * @param contact
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @throws sync4j.exchange.util.DataAccessException
     *
     */
    public void removeContact(Contact contact     ,
                              String  username    ,
                              String  credentials ,
                              String  exchangeFolder)
    throws DataAccessException {

        String webDavContactMsg        = null ;

        String webDavHeaderMsg         = null ;
        String response                = null ;

        String server                  = null ;
        String resource                = null ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        webDavContactMsg       = ""                      ;

        webDavHeaderMsg =
            MessageFormat.format(ItemDAO.WEBDAV_HEADER_REMOVE,
            new Object[] {
                        "/"                    +
                            server             +
                            "/"                +
                            username           +
                            "/"                +
                            resource             ,
                        contact.getHref()
            }
        );

        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg, credentials, webDavContactMsg, FILE_ENCODING);
        } catch (IOException e) {
            throw new DataAccessException("Error getting Exchange server response",
                                          e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                      +
                                          username                     +
                                          " URI "                      +
                                          exchangeFolder               +
                                          " removing exchange contact" ,
                                          e)                           ;

        }
    }

    /**
     * get contacts from Exchange Server
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     * @return array of find contacts
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact[] getContacts (String    username    ,
                                  String    credentials ,
                                  String[]  ids         ,
                                  String    exchangeFolder)
        throws DataAccessException {

        String clause = null ;

        clause = getClause(ids);

        return getContacts(username, credentials, clause, exchangeFolder);

        }

    /**
     * get contacts from Exchange Server
     *
     * @param username
     * @param credentials
     * @param fields
     * @param values
     * @param exchangeFolder
     * @return array of find contacts
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact[] getContacts (String username      ,
                                  String credentials   ,
                                  String fields[]      ,
                                  Object values[]      ,
                                  String exchangeFolder)
    throws DataAccessException {

        String clause = null;

        clause = getClause(fields, values);

        return getContacts(username, credentials, clause, exchangeFolder);
    }

     //------------------------------------------------------------- Private methods

    /**
     * Create a <i>Contact</i> array object from WebDav msg.
     *
     * @param id
     * @param webDavMsg
     *
     * @return a newly create <i>Calendar</i> object initialized with the
     *         fields in the result set
     *
     */
    private Contact[] getContactsFromWebDavMsg(String webDavMsg)
    throws DataAccessException {

        ArrayList  contacts   = null ;
        String []  resps      = null ;
        String []  msg        = null ;

        String     response   = null ;
        String     id         = null ;
        String     replUid    = null ;

        String     isFolder   = null ;

        int        count      = 0    ;

        msg = new String[] {webDavMsg};

        contacts = new ArrayList();

        try {

            resps = XmlParser.getXMLTag(msg, TAG_RESPONSE);

            for (int i = 0, l = resps.length; i < l; i++) {

                response    =  resps[i]                                        ;
                replUid     =  XmlParser.getXMLInitTagValue(response,
                                                        TAG_REPLUID)           ;

                id          =  getIdFromReplUid(replUid)                       ;

                isFolder    =  XmlParser.getXMLInitTagValue(response,
                                                            TAG_IS_FOLDER)     ;

                if(PROP_NO_FOLDER.equals(isFolder)) {
                    contacts.add(getContactFromResponseTag(id, response));
                    count ++;
                }

            }

        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing contact", e);
        }

        return (Contact[])contacts.toArray(new Contact[count]);

    }

    /**
     * Create a <i>Contact</i> object from response tag.
     *
     * @param id
     * @param msgResponse
     *
     * @return a newly create <i>Contact</i> object initialized with the
     *         fields in the result set
     *
     */
    private Contact getContactFromResponseTag(String id, String msgResponse)
    throws DataAccessException {

        Contact c           = null ;
        String  lastUpdate  = null ;

        String  value       = null ;

        PersonalDetail    personalDetail         = null ;
        BusinessDetail    businessDetail         = null ;
        Address           homeAddressBook        = null ;
        Address           workAddressBook        = null ;
        Address           otherAddressBook       = null ;

        Name              name                   = null ;
        Title             title                  = null ;
        Phone             phone                  = null ;
        Email             email                  = null ;
        WebPage           webPage                = null ;

        List              webPages               = null ;
        List              emails                 = null ;
        List              webPagesBusiness       = null ;
        List              emailsBusiness         = null ;
        List              phonesPersonal         = null ;
        List              phonesBusiness         = null ;

        String            response               = null ;

        StringBuffer      webDavContactMsg       = null ;
        String            webDavHeaderMsg        = null ;

        String            anniversary            = null ;
        String            birthday               = null ;
        String            categories             = null ;
        String            children               = null ;
        String            displayName            = null ;
        String            firstName              = null ;
        String            hobby                  = null ;
        String            importance             = null ;
        String            initials               = null ;
        String            lastName               = null ;
        String            language               = null ;
        String            middleName             = null ;
        String            mileage                = null ;
        String            nickName               = null ;
        String            sensitivity            = null ;
        String            spouse                 = null ;
        String            suffix                 = null ;
        String            salutation             = null ;

        String            yomiCompany            = null ;
        String            yomiFirstName          = null ;
        String            yomiLastName           = null ;

        String            homeStreet             = null ;
        String            homeCity               = null ;
        String            homeState              = null ;
        String            homePostOfficeBox      = null ;
        String            homePostalCode         = null ;
        String            homeCountry            = null ;

        String            officeStreet           = null ;
        String            officeCity             = null ;
        String            officeState            = null ;
        String            officeCountry          = null ;
        String            officePostalCode       = null ;
        String            officePostOfficeBox    = null ;

        String            otherStreet            = null ;
        String            otherCity              = null ;
        String            otherState             = null ;
        String            otherPostalCode        = null ;
        String            otherPostOfficeBox     = null ;
        String            otherCountry           = null ;

        String            assistantPhone         = null ;
        String            callBackPhone          = null ;
        String            homeFax                = null ;
        String            homePhone              = null ;
        String            homeOtherPhone         = null ;
        String            mobileOtherPhone       = null ;
        String            mobilePhone            = null ;
        String            officeFax              = null ;
        String            officeOtherPhone       = null ;
        String            officePhone            = null ;
        String            organizationPhone      = null ;
        String            otherFax               = null ;
        String            otherPhone             = null ;
        String            pager                  = null ;
        String            telexNumber            = null ;

        String            assistant              = null ;
        String            department             = null ;
        String            jobTitle               = null ;
        String            manager                = null ;
        String            officeLocation         = null ;
        String            organization           = null ;
        String            organizationIDNumber   = null ;
        String            profession             = null ;

        String            emailAddress1          = null ;
        String            emailAddress2          = null ;
        String            emailAddress3          = null ;
        String            email1AddressType      = null ;
        String            email2AddressType      = null ;
        String            email3AddressType      = null ;
        String            computerNetworkName    = null ;
        String            businessHomePage       = null ;
        String            personalHomePage       = null ;

        try {

            anniversary = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_ANNIVERSARY
                          );

            birthday  = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_BIRTHDAY
                        );

            categories
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_CATEGORIES
                          );

            children
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_CHILDREN
                          );

            displayName = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_FILEAS
                          );

            firstName = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_FIRSTNAME
                        );
            hobby = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_HOBBY
                    );

            importance = XmlParser.getXMLInitTagValue (
                          msgResponse,
                          TAG_IMPORTANCE
                         );

            initials = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_INITIALS
                       );

            language = XmlParser.getXMLInitTagValue (
                           msgResponse,
                           TAG_LANGUAGE
                       );

            lastName  = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_LASTNAME
                        );

            middleName = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_MIDDLENAME
                         );

            mileage
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_MILEAGE
            );

            nickName = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_NICKNAME
                       );

            sensitivity
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_SENSITIVITY
                        );

            spouse = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_SPOUSECN
                     );

            suffix = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_SUFFIX
            );

            salutation = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_TITLE
            );


            homeStreet =  XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_STREET
                          );

            homeCity = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_CITY
                       );

            homeState = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_STATE
                        );

            homePostalCode = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_HOME_POSTAL_CODE
                             );

            homePostOfficeBox
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_POST_OFFICE_BOX
            );

            homeCountry = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_COUNTRY
                          );


            officeStreet = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_WORK_STREET
                           );

            officeCity = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_WORK_CITY
                         );

            officeState = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_WORK_STATE
                          );

            officePostalCode = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_WORK_POSTALCODE
                               );

            officePostOfficeBox
                            = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_WORK_POST_OFFICE_BOX
                              );

            officeCountry = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_WORK_COUNTRY
                            );


            otherStreet = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OTHER_STREET
                          );

            otherCity = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OTHER_CITY
                        );

            otherState = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OTHER_STATE
                         );

            otherPostalCode = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_OTHER_POSTALCODE
                              );

            otherPostOfficeBox
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OTHER_POST_OFFICE_BOX
            );

            otherCountry = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OTHER_COUNTRY
                           );


            assistant = XmlParser.getXMLInitTagValue (
                              msgResponse,
                              TAG_SECRETARY
                          );

            department = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_DEPARTMENT
                         );

            jobTitle = XmlParser.getXMLInitTagValue (
                           msgResponse,
                           TAG_JOBTITLE
                       );

            manager = XmlParser.getXMLInitTagValue (
                         msgResponse,
                         TAG_MANAGER
                       );

            officeLocation
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OFFICE_LOCATION
            );

            organization =  XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_ORGANIZATION
                            );

            organizationIDNumber
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_ORGANIZATION_ID_NUMBER
            );

            profession = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_PROFESSION
                         );


            assistantPhone = XmlParser.getXMLInitTagValue (
                                 msgResponse,
                                 TAG_SECRETARY_PHONE
                              );


            callBackPhone = XmlParser.getXMLInitTagValue (
                                 msgResponse,
                                 TAG_CALL_BACK_PHONE
                             );


            homeFax = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_HOME_FAX
                      );

            homeOtherPhone = XmlParser.getXMLInitTagValue (
                                 msgResponse,
                                 TAG_HOME_PHONE2
                             );


            homePhone = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_HOME_PHONE
                         );

            mobileOtherPhone = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_OTHER_MOBILE
                               );


            mobilePhone = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_MOBILE
                          );

            officeFax = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_FAC_SIMILE_TELEPHONE_NUMBER
                        );


            officePhone = XmlParser.getXMLInitTagValue (
                             msgResponse,
                             TAG_TELEPHONE_NUMBER
                           );

            officeOtherPhone = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_TELEPHONE_NUMBER2
                               );

            organizationPhone = XmlParser.getXMLInitTagValue (
                                    msgResponse,
                                    TAG_ORGANIZATION_MAIN_PHONE
                                );


            otherFax = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_OTHER_FAX
                       );

            otherPhone = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_OTHER_TELEPHONE
                         );


            pager = XmlParser.getXMLInitTagValue (
                        msgResponse,
                        TAG_PAGER
                    );

            telexNumber
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_TELEX_NUMBER
            );

            value =
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL1
                );

            if (value != null && value.trim().length() > 0) {

                value = value.replaceAll("&lt;", "<");
                value = value.replaceAll("&gt;", ">");

                if (value.indexOf("<") != -1 && value.indexOf(">") != -1) {

                    emailAddress1 = value.substring(value.indexOf("<") + 1,
                                                    value.indexOf(">"));
                } else {
                    emailAddress1 = value;
                }

            }

            value =
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL2
                );

            if (value != null && value.trim().length() > 0) {

                value = value.replaceAll("&lt;", "<");
                value = value.replaceAll("&gt;", ">");

                if (value.indexOf("<") != -1 && value.indexOf(">") != -1) {

                    emailAddress2 = value.substring(value.indexOf("<") + 1,
                                                        value.indexOf(">"));
                } else {
                    emailAddress2 = value;

                }
            }

            value =
                XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL3
                );

            if (value != null && value.trim().length() > 0) {

                value = value.replaceAll("&lt;", "<");
                value = value.replaceAll("&gt;", ">");

                if (value.indexOf("<") != -1 && value.indexOf(">") != -1) {

                    emailAddress3 = value.substring(value.indexOf("<") + 1,
                                                        value.indexOf(">"));
                } else {
                    emailAddress3 = value;

                }
            }

            email1AddressType
                = XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL1_ADDRESS_TYPE
                );

            email2AddressType
                = XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL2_ADDRESS_TYPE
                );

            email3AddressType
                = XmlParser.getXMLInitTagValue (
                    msgResponse,
                    TAG_EMAIL3_ADDRESS_TYPE
                );

            businessHomePage = XmlParser.getXMLInitTagValue (
                                msgResponse,
                                TAG_BUSINESS_HOME_PAGE
                             );

            personalHomePage
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_PERSONAL_HOME_PAGE
            );

            computerNetworkName
                        = XmlParser.getXMLInitTagValue (
                            msgResponse,
                            TAG_COMPUTER_NETWORK_NAME
            );

            lastUpdate =
                XmlParser.getXMLInitTagValue(msgResponse,
                TAG_LAST_MODIFIED);

            c = new Contact(id, XmlParser.webDavTagToDate(lastUpdate));

            if (firstName != null) {
                c.getName().getFirstName().setPropertyValue(firstName);
            }

            if (lastName != null) {
                c.getName().getLastName().setPropertyValue(lastName);
            }

            if (middleName != null) {
                c.getName().getMiddleName().setPropertyValue(middleName);
            }

            if (nickName != null) {
                c.getName().getNickname().setPropertyValue(nickName);
            }

            if (displayName != null) {
                c.getName().getDisplayName().setPropertyValue(displayName);
            }

            if (suffix != null) {
                c.getName().getSuffix().setPropertyValue(suffix);
            }

            if (importance != null) {
                c.setImportance(new Short(importance));
            }

            if (initials != null) {
                c.getName().getInitials().setPropertyValue(initials);
            }

            if (categories != null) {
                c.setCategories(XmlParser.getFieldFromMultipleTag(categories));
            }

            if (salutation != null) {
                c.getName().getSalutation().setPropertyValue(salutation);
            }

            if (mileage != null) {
                c.setMileage(new Integer (mileage));
            }

            if (sensitivity != null) {
                c.setSensitivity(new Short(sensitivity));
            }

            if (jobTitle != null) {
                title = new Title();
                title.setTitleType(FIELD_TYPE_JOB_TITLE);
                title.setPropertyValue(jobTitle);
                c.getBusinessDetail().addTitle(title);
            }

            if (organization != null) {
                c.getBusinessDetail().getCompany().setPropertyValue(organization);
            }

            if (department != null) {
                c.getBusinessDetail().getDepartment().setPropertyValue(department);
            }

            if (profession != null) {
                c.getBusinessDetail().getRole().setPropertyValue(profession);
            }

            if (anniversary != null) {
                c.getPersonalDetail().
                    setAnniversary(XmlParser.
                        webDavToPDIAnniversary(anniversary));
            }

            if (birthday != null) {
                c.getPersonalDetail().
                    setBirthday(XmlParser.
                        webDavToPDIAnniversary(birthday));
            }

            if (children != null) {
                c.getPersonalDetail().
                    setChildren(XmlParser.
                        getFieldFromMultipleTag(children));
            }

            if (hobby != null) {
                c.getPersonalDetail().setHobbies(hobby);
            }

            if (language != null) {
                c.setLanguages(language);
            }

            if (spouse != null) {
                c.getPersonalDetail().setSpouse(spouse);
            }

            if (manager != null) {
                c.getBusinessDetail().setManager(manager);
            }

            if (assistant != null) {
                c.getBusinessDetail().setAssistant(assistant);
            }

            if (officeLocation != null) {
                c.getBusinessDetail().setOfficeLocation(officeLocation);
            }

            if (officePhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_BUSINESS_TELEPHONE_NUMBER);
                phone.setPropertyValue(officePhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (homePhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_HOME_TELEPHONE_NUMBER);
                phone.setPropertyValue(homePhone);
                c.getPersonalDetail().addPhone(phone);
            }

            if (mobilePhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_MOBILE_TELEPHONE_NUMBER);
                phone.setPropertyValue(mobilePhone);
                c.getPersonalDetail().addPhone(phone);
            }

            if (assistantPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_ASSISTANT_NUMBER);
                phone.setPropertyValue(assistantPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (officeOtherPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_BUSINESS_OTHER_TELEPHONE_NUMBER);
                phone.setPropertyValue(officeOtherPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (officeFax != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_BUSINESS_FAX_NUMBER);
                phone.setPropertyValue(officeFax);
                c.getBusinessDetail().addPhone(phone);
            }

            if (callBackPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_CALLBACK_NUMBER);
                phone.setPropertyValue(callBackPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (mobileOtherPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_CAR_TELEPHONE_NUMBER);
                phone.setPropertyValue(mobileOtherPhone);
                c.getPersonalDetail().addPhone(phone);
            }

            if (organizationPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_COMPANY_MAIN_TELEPHONE_NUMBER);
                phone.setPropertyValue(organizationPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (homeOtherPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_HOME_OTHER_TELEPHONE_NUMBER);
                phone.setPropertyValue(homeOtherPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (homeFax != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_HOME_FAX_NUMBER);
                phone.setPropertyValue(homeFax);
                c.getBusinessDetail().addPhone(phone);
            }

            if (otherPhone != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_OTHER_TELEPHONE_NUMBER);
                phone.setPropertyValue(otherPhone);
                c.getBusinessDetail().addPhone(phone);
            }

            if (otherFax != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_OTHER_FAX_NUMBER);
                phone.setPropertyValue(otherFax);
                c.getBusinessDetail().addPhone(phone);
            }

            if (pager != null) {
                phone = new Phone();
                phone.setPhoneType(FIELD_TYPE_PAGER_NUMBER);
                phone.setPropertyValue(pager);
                c.getBusinessDetail().addPhone(phone);
            }

            if (businessHomePage != null) {
                webPage = new WebPage();
                webPage.setWebPageType(FIELD_TYPE_BUSINESS_HOME_PAGE);
                webPage.setPropertyValue(businessHomePage);
                c.getBusinessDetail().addWebPage(webPage);
            }

            if (personalHomePage != null) {
                webPage = new WebPage();
                webPage.setWebPageType(FIELD_TYPE_PERSONAL_HOME_PAGE);
                webPage.setPropertyValue(personalHomePage);
                c.getPersonalDetail().addWebPage(webPage);
            }

            if (emailAddress1 != null) {
                email = new Email();
                email.setEmailType(FIELD_TYPE_EMAIL1_ADDRESS);
                email.setPropertyValue(emailAddress1);
                c.getPersonalDetail().addEmail(email);
            }

            if (emailAddress2 != null) {
                email = new Email();
                email.setEmailType(FIELD_TYPE_EMAIL2_ADDRESS);
                email.setPropertyValue(emailAddress2);
                c.getPersonalDetail().addEmail(email);
            }

            if (emailAddress3 != null) {
                email = new Email();
                email.setEmailType(FIELD_TYPE_EMAIL3_ADDRESS);
                email.setPropertyValue(emailAddress3);
                c.getPersonalDetail().addEmail(email);
            }

            if (homeStreet != null) {
                c.getPersonalDetail().getAddress().
                                        getStreet().
                                            setPropertyValue(homeStreet);
            }

            if (homeCity != null) {
                c.getPersonalDetail().getAddress().
                                        getCity().
                                            setPropertyValue(homeCity);
            }

            if (homeState != null) {
                c.getPersonalDetail().getAddress().
                                        getState().
                                            setPropertyValue(homeState);
            }

            if (homePostalCode != null) {
                c.getPersonalDetail().getAddress().
                                        getPostalCode().
                                            setPropertyValue(homePostalCode);
            }

            if (homePostOfficeBox != null) {
                c.getPersonalDetail().getAddress().
                                        getPostOfficeAddress().
                                            setPropertyValue(homePostOfficeBox);
            }

            if (homeCountry != null) {
                c.getPersonalDetail().getAddress().
                                        getCountry().
                                            setPropertyValue(homeCountry);
            }


            if (otherStreet != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getStreet().
                                            setPropertyValue(otherStreet);
            }

            if (otherCity != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getCity().
                                            setPropertyValue(otherCity);
            }

            if (otherState != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getState().
                                            setPropertyValue(otherState);
            }

            if (otherPostalCode != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getPostalCode().
                                            setPropertyValue(otherPostalCode);
            }

            if (homePostOfficeBox != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getPostOfficeAddress().
                                            setPropertyValue(homePostOfficeBox);
            }


            if (otherCountry != null) {
                c.getPersonalDetail().getOtherAddress().
                                        getCountry().
                                            setPropertyValue(otherCountry);
            }


            if (officeStreet != null) {
                c.getBusinessDetail().getAddress().
                                        getStreet().
                                            setPropertyValue(officeStreet);
            }

            if (officeCity != null) {
                c.getBusinessDetail().getAddress().
                                        getCity().
                                            setPropertyValue(officeCity);
            }

            if (officeState != null) {
                c.getBusinessDetail().getAddress().
                                        getState().
                                            setPropertyValue(officeState);
            }

            if (officePostalCode != null) {
                c.getBusinessDetail().getAddress().
                                        getPostalCode().
                                            setPropertyValue(officePostalCode);
            }

            if (officePostOfficeBox != null) {
                c.getBusinessDetail().getAddress().
                                        getPostOfficeAddress().
                                            setPropertyValue(officePostOfficeBox);
            }

            if (officeCountry != null) {
                c.getBusinessDetail().getAddress().
                                        getCountry().
                                            setPropertyValue(officeCountry);
            }



        } catch (Exception e) {
            throw new DataAccessException("Error parsing contact item", e);
        }

        return c;

    }


    /**
     * get contacts from Exchange Server with the given clause
     *
     * @param username
     * @param credentials
     * @param clause
     * @param exchangeFolder
     * @return array of find contacts
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    private Contact[] getContacts (String    username    ,
                                   String    credentials ,
                                   String    clause      ,
                                   String    exchangeFolder)
            throws DataAccessException {

        Contact[] exchangeContacts    = null ;

        String response               = null ;
        String webDavContactMsg       = null ;
        String webDavHeaderMsg        = null ;

        String server                 = null ;
        String resource               = null ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        webDavContactMsg =
            MessageFormat.format(WEBDAV_MSG_SELECT_CONTACTS,
            new Object[] {server, username, resource, clause});

        webDavHeaderMsg =
            MessageFormat.format(ItemDAO.WEBDAV_HEADER_SELECT,

            new Object[] {
                        "/"           +
                            server    +
                            "/"       +
                            username  +
                            "/"       +
                        resource

            }
        );


        try {
             response = this.webDavHttp.sendRequest(webDavHeaderMsg, credentials, webDavContactMsg, FILE_ENCODING);
        } catch (IOException e) {
             throw new DataAccessException("Error getting Exchange server response",
                                           e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                       +
                                          username                      +
                                          " URI "                       +
                                          exchangeFolder                +
                                          " getting Exchange contact"   ,
                                          e)                            ;

        }

        exchangeContacts = getContactsFromWebDavMsg(response);

        return exchangeContacts;

    }

    /**
     * Build a clause
     * to webdav select request
     * between ids item array
     *
     * @param ids
     * @return webdav select clause
     **/
    private static String getClause (String[] fields, Object[] values) {

        StringBuffer clause = new StringBuffer();

        int l = fields.length;

        if (l == 0) {
            return "";
        }

        clause.append("where ");

        StringBuffer tmp = null;
        String valueTmp  = null;

        for (int i = 0; i < l; i++) {
            tmp = new StringBuffer();
            valueTmp = String.valueOf(values[i]);
            valueTmp = valueTmp.replaceAll("'", "''");
            valueTmp = StringTools.escapeXml(valueTmp);

            if (valueTmp == null || "null".equals(valueTmp)) {
                valueTmp = "";
            }

            if (TAG_FIRSTNAME.equals(fields[i])) {
                tmp.append("\"").append(FIELD_PREFIX).append("givenName\"");
                if (!valueTmp.equals("")) {
                    tmp.append("='").append(valueTmp).append("'");
                } else {
                    tmp.append(" is null ");
                }
            }

            if (TAG_LASTNAME.equals(fields[i])) {
                tmp.append("\"").append(FIELD_PREFIX).append("sn\"");
                if (!valueTmp.equals("")) {
                    tmp.append("='").append(valueTmp).append("'");
                } else {
                    tmp.append(" is null ");
                }
            }

            if (TAG_MIDDLENAME.equals(fields[i])) {
                tmp.append("\"").append(FIELD_PREFIX).append("middlename\"");
                if (!valueTmp.equals("")) {
                    tmp.append("='").append(valueTmp).append("'");
                } else {
                    tmp.append(" is null ");
                }
            }

            if (TAG_EMAIL1.equals(fields[i])) {
                tmp.append("\"").append(FIELD_PREFIX).append("email1\"");
                if (!valueTmp.equals("")) {
                    tmp.append("='").append(valueTmp).append("'");
                } else {
                    tmp.append(" is null ");
                }
            }

            if (i != 0 && tmp.length() != 0) {
                clause.append(" and ").append(tmp);
            } else if (tmp.length() != 0) {
                clause.append(tmp);
            }

        }

        clause.append("\r\n");

        return clause.toString();

    }

}
