/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.contact.manager;

import java.io.IOException;

import java.util.List;

import sync4j.exchange.items.contact.dao.ContactDAO;
import sync4j.exchange.items.contact.model.Contact;
import sync4j.exchange.DataAccessException;

import sync4j.foundation.pdi.contact.ContactDetail;
import sync4j.foundation.pdi.contact.Email;
import sync4j.foundation.pdi.contact.PersonalDetail;

/*
 * This class defines methods to access contacts data
 * in exchange server datastore
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: ContactManager.java,v 1.8 2005/06/20 14:04:02 fabius Exp $
 *
 **/
public class ContactManager {

    //------------------------------------------------------------- Constants

    //------------------------------------------------------------- Private Data

    ContactDAO cd = null ;

    //------------------------------------------------------------- Costructor

    public ContactManager(String exchangeServerHost ,
                          int    exchangeServerPort )
        throws DataAccessException {

        this.cd = new ContactDAO (exchangeServerHost ,
                                  exchangeServerPort);
    }

    //------------------------------------------------------------- Public methods

    /**
     * get all contacts
     *
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return array of find contacts
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact[] getAllContacts(String   username    ,
                                    String   credentials ,
                                    String   exchangeFolder   )
        throws DataAccessException {

        return this.cd.getContacts(username      ,
                                   credentials   ,
                                   new String[0] ,
                                   exchangeFolder     );

    }

    /**
     * get contacts
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find contacts
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact[] getContacts(String   username    ,
                                 String   credentials ,
                                 String[] ids         ,
                                 String   exchangeFolder   )
        throws DataAccessException {

        return this.cd.getContacts(username    ,
                                   credentials ,
                                   ids         ,
                                   exchangeFolder   );

    }

    /**
     * get contact by key
     *
     * @param id
     * @param username
     * @param credentials
     *
     * @return find contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact getContactById(String username    ,
                                  String credentials ,
                                  String id          ,
                                  String exchangeFolder   )
        throws DataAccessException {

        Contact[] contacts = null;

        contacts = this.cd.getContacts(username          ,
                                       credentials       ,
                                       new String[] {id} ,
                                       exchangeFolder         );

        if (contacts == null || !(contacts.length > 0)) {
            return null;
        }

        return contacts[0];

    }

    /**
     * get contact twin
     *
     * @param contact the Contact object
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return find contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact getContactTwin(Contact contact       ,
                                   String username      ,
                                   String credentials   ,
                                   String exchangeFolder)
    throws DataAccessException {

        Contact[] contacts = null;


        String          firstName              = null ;
        String          lastName               = null ;
        String          middleName             = null ;

        String          emailAddress           = null ;

        List            emails                 = null ;

        Email           email                  = null ;

        PersonalDetail  personalDetail         = null ;
        ContactDetail   personalContactDetail  = null ;

        personalDetail = contact.getPersonalDetail();

        if (personalDetail != null) {
            emails = personalDetail.getEmails();
        }

        if (emails != null && emails != null) {

            for (int i=0, l = emails.size(); i < l; i++) {

                email = (Email) emails.get(i);

                if ((ContactDAO.FIELD_TYPE_EMAIL1_ADDRESS).equals(email.getEmailType())) {
                    emailAddress = email.getPropertyValueAsString();
                }

            }

        }

        if (contact.getName().getFirstName() != null) {
            firstName = contact.getName().
                            getFirstName ().
                                getPropertyValueAsString();
        }
        if (contact.getName().getLastName() != null) {
            lastName  = contact.getName().
                                getLastName().
                                    getPropertyValueAsString();
        }
        if (contact.getName().getMiddleName() != null) {
            middleName  = contact.getName().
                                getMiddleName().
                                    getPropertyValueAsString();
        }

        String[] fields = {
                          ContactDAO.TAG_FIRSTNAME,
                          ContactDAO.TAG_LASTNAME,
                          ContactDAO.TAG_MIDDLENAME,
                          ContactDAO.TAG_EMAIL1
        };

        Object[] values = {
                          firstName,
                          lastName,
                          middleName,
                          emailAddress
        };


        contacts = this.cd.getContacts(username          ,
                                       credentials       ,
                                       fields            ,
                                       values            ,
                                       exchangeFolder    );

        if (contacts == null || !(contacts.length > 0)) {
            return null;
        }

        return contacts[0];
    }

    /**
     * add contact
     *
     * @param contact
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact setContact(Contact contact     ,
                              String  username    ,
                              String  credentials ,
                              String  exchangeFolder   )
        throws DataAccessException {

        return this.cd.setContact(contact     ,
                                  username    ,
                                  credentials ,
                                  exchangeFolder   );

    }

    /**
     * remove contact
     *
     * @param contact
     * @param username
     * @param principal
     * @param exchangeFolder
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void removeContact(Contact contact   ,
                              String  username  ,
                              String  principal ,
                              String  exchangeFolder )
        throws DataAccessException {

        this.cd.removeContact(contact   ,
                              username  ,
                              principal ,
                              exchangeFolder );

    }

}
