/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.task.dao;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;

import sync4j.exchange.util.StringTools;

import sync4j.exchange.httptransport.WebDavHttpTransport;
import sync4j.exchange.items.task.model.Task;
import sync4j.exchange.items.common.dao.ItemDAO;
import sync4j.exchange.DataAccessException;
import sync4j.exchange.xml.XmlParseException;
import sync4j.exchange.xml.XmlParser;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.TimeZone;


/*
 * This class implements methods to access task data
 * in exchange server datastore
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: TaskDAO.java,v 1.16 2005/06/27 13:49:28 nichele Exp $
 *
 **/
public class TaskDAO extends ItemDAO {

    //---------------------------------------------------------------- Constants

    private static final String WEBDAV_MSG_SELECT_TASKS                       =
    "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"                           +
    "<D:searchrequest xmlns:D =\"DAV:\">\n"                                   +
    "<D:sql>\n"                                                               +
    "Select "                                                                 +
    "\"urn:schemas:httpmail:textdescription\" AS textdescription, "           +
    "\"urn:schemas:httpmail:subject\" AS subject, "                           +
    "\"urn:schemas:calendar:dtstart\" AS dtstart, "                           +
    "\"urn:schemas:calendar:dtend\" AS dtend, "                               +
    "\"http://schemas.microsoft.com/exchange/keywords-utf8\" AS categories, " +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8105\" as date, " +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8105\" as duedate, " +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8102\" as perc, " +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x8101\" as status, " +
    "\"http://schemas.microsoft.com/mapi/id/'{'00062003-0000-0000-C000-000000000046'}'/0x811c\" as complete, " +
    "\"http://schemas.microsoft.com/mapi/reminderset\" as reminderset, "      +
    "\"http://schemas.microsoft.com/mapi/remindertime\" as remindertime, "    +
    "\"urn:schemas:httpmail:importance\" as importance, "                     +
    "\"http://schemas.microsoft.com/exchange/mileage\" as mileage, "          +
    "\"http://schemas.microsoft.com/mapi/sensitivity\" as sensitivity, "      +
    "\"http://schemas.microsoft.com/repl/repl-uid\" AS repluid, "             +
    "\"DAV:getlastmodified\" AS getlastmodified, "                            +
     "\"DAV:isfolder\" AS isfolder "                                          +
    "FROM \"/{0}/{1}/{2}\"  "                                                 +
    "{3} "                                                                    +
    "</D:sql>\n"                                                              +
    "</D:searchrequest>"                                                      ;

    public static final String TAG_DATE               = "date"             ;
    public static final String TAG_DESCRIPTION        =
        "textdescription"                                                   ;

    public static final String TAG_RESPONSE           = "a:response"       ;
    public static final String TAG_SUBJECT            = "subject"          ;

    public static final String TAG_REPLUID            = "repluid"          ;

    public static final String TAG_LAST_MODIFIED      = "getlastmodified"  ;

    public static final String TAG_DUE_DATE           = "duedate"          ;


    //------------------------------------------------------------- Private data

    private WebDavHttpTransport  webDavHttp          = null ;

    //------------------------------------------------------------- Costructors

    public TaskDAO(String exchangeServerHost ,
                   int    exchangeServerPort)
        throws DataAccessException {

        this.webDavHttp         = new WebDavHttpTransport(exchangeServerHost,
                                                          exchangeServerPort);
    }
    //------------------------------------------------------------- Public methods

    /**
     * insert / update task
     *
     * @param task
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @return new / updated task
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task setTask(Task    task           ,
                        String  username       ,
                        String  credentials    ,
                        String  exchangeFolder )
        throws DataAccessException {


        String response                = null ;

        String id                      = null ;

        StringBuffer webDavTaskMsg     = new StringBuffer();

        String webDavHeaderMsg         = null ;

        String server                  = null ;
        String resource                = null ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );


        webDavTaskMsg.append("<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n");
        webDavTaskMsg.append("<D:propertyupdate xmlns:D=\"DAV:\" ");
        webDavTaskMsg.append("xmlns:EX=\"http://schemas.microsoft.com/exchange/\" ");
        webDavTaskMsg.append("xmlns:HN=\"urn:schemas:httpmail:\" ");
        webDavTaskMsg.append("xmlns:MAPI_ID=\"http://schemas.microsoft.com/mapi/id/{00062003-0000-0000-C000-000000000046}/\" ");
        webDavTaskMsg.append("xmlns:MAPI=\"http://schemas.microsoft.com/mapi/\" ");
        webDavTaskMsg.append("xmlns:x=\"xml:\" ");
        webDavTaskMsg.append("xmlns:b=\"urn:uuid:c2f41010-65b3-11d1-a29f-00aa00c14882/\">\n");
        webDavTaskMsg.append("<D:set>\n");
        webDavTaskMsg.append(" <D:prop>\n");
        webDavTaskMsg.append("  <D:contentclass>urn:content-classes:person</D:contentclass>\n");
        webDavTaskMsg.append("  <EX:outlookmessageclass>IPM.Task</EX:outlookmessageclass>\n");

        String subject = task.getSubject();

        if (subject != null) {
            subject  = StringTools.escapeXml(subject);
            webDavTaskMsg.append("<HN:subject>");
            webDavTaskMsg.append(subject);
            webDavTaskMsg.append("</HN:subject>");
        }

        String textDescription = null;

        textDescription = task.getTextDescription();

        if (textDescription != null) {
            textDescription  = StringTools.escapeXml(textDescription);
            webDavTaskMsg.append("<HN:textdescription>");
            webDavTaskMsg.append(textDescription);
            webDavTaskMsg.append("</HN:textdescription>");
        }

        String tmp = dateToWebDavTag(task.getDate());
        if (tmp != null && !tmp.equals("")) {
            webDavTaskMsg.append("<MAPI_ID:0x8104 b:dt=\"dateTime.tz\">");
            webDavTaskMsg.append(tmp);
            webDavTaskMsg.append("</MAPI_ID:0x8104>");
        }

        tmp = dateToWebDavTag(task.getDueDate());
        if (tmp != null && !tmp.equals("")) {
            webDavTaskMsg.append("<MAPI_ID:0x8105 b:dt=\"dateTime.tz\">");
            webDavTaskMsg.append(tmp);
            webDavTaskMsg.append("</MAPI_ID:0x8105>");
        }

        if (task.getImportance() != null) {
            webDavTaskMsg.append("<HN:importance>");
            webDavTaskMsg.append(task.getImportance());
            webDavTaskMsg.append("</HN:importance>");
        }

        if (task.getMileage() != null) {
            webDavTaskMsg.append("<EX:mileage>");
            webDavTaskMsg.append(task.getMileage());
            webDavTaskMsg.append("</EX:mileage>");
        }

        String categories = task.getCategories();

        if (categories != null && categories.length() > 0) {

            categories  = StringTools.escapeXml(categories);

            categories = XmlParser.
                            fieldToMultipleTag
                                (categories);

            webDavTaskMsg.append("<EX:keywords-utf8>");
            webDavTaskMsg.append(categories);
            webDavTaskMsg.append("</EX:keywords-utf8>");
        }

        String sPerc = task.getPercentComplete();
        if (sPerc != null) {
            float perc = Float.parseFloat(sPerc) / 100;
            webDavTaskMsg.append("<MAPI_ID:0x8102 b:dt=\"float\">");
            webDavTaskMsg.append(perc);
            webDavTaskMsg.append("</MAPI_ID:0x8102>");
        }


        webDavTaskMsg.append("<MAPI:reminderset>");
        webDavTaskMsg.append((task.getReminderSet() ? "1" : "0"));
        webDavTaskMsg.append("</MAPI:reminderset>");

        tmp = dateToWebDavTag(task.getReminderTime());
        if (tmp != null && !tmp.equals("")) {
            webDavTaskMsg.append("<MAPI:remindertime b:dt=\"dateTime.tz\">");
            webDavTaskMsg.append(tmp);
            webDavTaskMsg.append("</MAPI:remindertime>");
        }

        if (task.getSensitivity() != null) {
            webDavTaskMsg.append("<MAPI:sensitivity>");
            webDavTaskMsg.append(task.getSensitivity());
            webDavTaskMsg.append("</MAPI:sensitivity>");
        }

        if (task.getTextDescription() != null) {
            webDavTaskMsg.append("<MAPI_ID:0x8101  b:dt=\"int\">");
            webDavTaskMsg.append(task.getStatus());
            webDavTaskMsg.append("</MAPI_ID:0x8101>");
        }

        webDavTaskMsg.append("<MAPI_ID:0x811c b:dt=\"boolean\">");
        webDavTaskMsg.append( (task.isComplete() ? "1" : "0") );
        webDavTaskMsg.append("</MAPI_ID:0x811c>");

        webDavTaskMsg.append(" </D:prop>\n");
        webDavTaskMsg.append("</D:set>\n");
        webDavTaskMsg.append("</D:propertyupdate>");

        webDavHeaderMsg
            = MessageFormat.format(ItemDAO.WEBDAV_HEADER_PROPPATCH,
                new Object[] {
                    "/"                   +
                        server            +
                        "/"               +
                        username          +
                        "/"               +
                        resource             ,
                    task.getHref()
              }
        );


        try {
             response = this.webDavHttp.sendRequest(webDavHeaderMsg,
                                                    credentials,
                                                    webDavTaskMsg.toString(),
                                                    FILE_ENCODING);
        } catch (Exception e) {
            throw new DataAccessException("Error getting Exchange server response",
                                          e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                   +
                                          username                  +
                                          " URI "                   +
                                          exchangeFolder            +
                                          " setting exchange task"  ,
                                          e)                        ;

        }

        try {
            id = XmlParser.getRuidFromResponse(response);
        } catch (XmlParseException e) {
            throw new DataAccessException("Error getting task id", e);
        }

        task.setId(id);

        return task;

    }

    /**
     * Delete a <i>Task</i> from Exchange server.
     *
     * @param task
     * @param username
     * @param credentials
     * @param exchangeFolder
     *
     * @throws sync4j.exchange.util.DataAccessException
     *
     */
    public void removeTask(Task    task           ,
                           String  username       ,
                           String  credentials    ,
                           String  exchangeFolder )
        throws DataAccessException {

        String webDavTaskMsg           = null ;

        String webDavHeaderMsg         = null ;

        String response                = null ;

        String server                  = null ;
        String resource                = null ;

        server   = getServerFromExchangeFolder   (exchangeFolder );
        resource = getResourceFromExchangeFolder (exchangeFolder );

        webDavTaskMsg          = ""                ;

        webDavHeaderMsg =
            MessageFormat.format(ItemDAO.WEBDAV_HEADER_REMOVE,
            new Object[] {
                        "/"                     +
                            server              +
                            "/"                 +
                            username            +
                            "/"                 +
                            resource              ,
                        task.getHref()
            }
        );

        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg, credentials, webDavTaskMsg, FILE_ENCODING);
        } catch (IOException e) {
            throw new DataAccessException("Error getting Exchange server response",
                                          e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER "                     +
                                          username                    +
                                          " URI "                     +
                                          exchangeFolder              +
                                          " removing exchange task"   ,
                                          e)                          ;

        }

    }

    /**
     * get tasks from Exchange Server
     *
     * @param username
     * @param credentials
     * @param ids
     * @param exchangeFolder
     *
     * @return array of find tasks
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task[] getTasks (String    username       ,
                            String    credentials    ,
                            String[]  ids            ,
                            String    exchangeFolder )
        throws DataAccessException {

        String clause        = getClause(ids);
        return getTasks(username, credentials, clause, exchangeFolder);
    }


    /**
     * get task array from Exchange Server
     *
     * @param username
     * @param credentials
     * @param fields
     * @param values
     * @param exchangeFolder
     * @return array of find tasks
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Task[] getTasks(String username,
                           String credentials,
                           String fields[],
                           Object values[],
                           String exchangeFolder) throws DataAccessException {

        String clause                 = null ;

        clause = getClause(fields, values);

        return getTasks(username, credentials, clause, exchangeFolder);
    }

    //------------------------------------------------------------- Private methods

    /**
     * Create a <i>Task</i> array object from WebDav msg.
     *
     * @param id
     * @param webDavMsg
     *
     * @return a newly create <i>Task</i> object initialized with the
     *         fields in the result set
     *
     */
    private Task[] getTasksFromWebDavMsg(String webDavMsg)
    throws DataAccessException {

        ArrayList  tasks      = null ;
        String []  resps      = null ;
        String []  msg        = null ;

        String     response   = null ;
        String     id         = null ;
        String     replUid    = null ;

        String     isFolder   = null ;
        int        count      = 0    ;

        msg = new String[] {webDavMsg};

        tasks = new ArrayList();

        try {

            resps = XmlParser.getXMLTag(msg, TAG_RESPONSE);

            for (int i = 0, l = resps.length; i < l; i++) {

                response =  resps[i]                                           ;
                replUid  =  XmlParser.getXMLInitTagValue(response, TAG_REPLUID);
                id       =  getIdFromReplUid(replUid)                          ;

                isFolder    =  XmlParser.getXMLInitTagValue(response,
                                                            TAG_IS_FOLDER)  ;
                if(PROP_NO_FOLDER.equals(isFolder)) {
                    tasks.add(getTaskFromResponseTag(id, response));
                    count ++;
                }

            }

        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing task", e);
        }

        return (Task[])tasks.toArray(new Task[count]);

    }

    /**
     * Create a <i>Task</i> object from response tag.
     *
     * @param id
     * @param msgResponse
     *
     * @return a newly create <i>Task</i> object initialized with the
     *         fields in the result set
     *
     */
    private Task getTaskFromResponseTag(String id, String msgResponse)
    throws DataAccessException {

        Task   t            = null ;
        String lastUpdate   = null ;
        String date         = null ;

        String categories   = null;

        t = new Task(id);

        try {

            t.setTextDescription(XmlParser.getXMLInitTagValue
                                 (msgResponse, TAG_DESCRIPTION));

            t.setSubject(XmlParser.getXMLInitTagValue
                         (msgResponse, TAG_SUBJECT));

            date = XmlParser.getXMLInitTagValue
                   (msgResponse, TAG_DATE);

            t.setDate(XmlParser.webDavTagToDate(date));

            lastUpdate = XmlParser.getXMLInitTagValue
                         (msgResponse, TAG_LAST_MODIFIED);

            t.setLastModified(XmlParser.webDavTagToDate
                              (lastUpdate));

            t.setTextDescription(XmlParser.getXMLInitTagValue
                                 (msgResponse, TAG_DESCRIPTION));

            categories = XmlParser.getXMLInitTagValue
                            (msgResponse, "categories");

            if (categories != null) {

                try {
                    categories = XmlParser.getFieldFromMultipleTag(categories);
                } catch (Exception e) {
                    throw new DataAccessException(e.getMessage());
                }

                t.setCategories(categories);

            }

            t.setDueDate(XmlParser.webDavTagToDate(XmlParser.getXMLInitTagValue
                                                   (msgResponse, "duedate")));

            t.setImportance(XmlParser.getXMLInitTagValue
                            (msgResponse, "importance"));

            t.setMileage(XmlParser.getXMLInitTagValue
                         (msgResponse, "mileage"));

            t.setPercentComplete(XmlParser.getXMLInitTagValue
                                 (msgResponse, "perc"));

            String tmp = XmlParser.getXMLInitTagValue
                             (msgResponse, "reminderset");

            if (tmp != null && tmp.equals("1")) {
                t.setReminderSet(true);
            } else {
                t.setReminderSet(false);
            }

            t.setReminderTime(XmlParser.webDavTagToDate(XmlParser.getXMLInitTagValue
                                                        (msgResponse, "remindertime")));

            t.setSensitivity(XmlParser.getXMLInitTagValue
                             (msgResponse, "sensitivity"));

            t.setStatus(XmlParser.getXMLInitTagValue
                        (msgResponse, "status"));

            tmp = XmlParser.getXMLInitTagValue(msgResponse, "complete");

            if (tmp != null && tmp.equals("1")) {
                t.setComplete(true);
            } else {
                t.setComplete(false);
            }

        } catch (XmlParseException e) {
            throw new DataAccessException("Error parsing task item: ", e);
        }

        return t;

    }

    /**
     * Build a clause
     * to webdav select request
     * between ids item array
     *
     * @param ids
     * @return webdav select clause
     **/
    private static String getClause (String[] fields, Object[] values) {

        StringBuffer clause = new StringBuffer();

        int l = fields.length;

        if (l == 0) {
            return "";
        }

        clause.append("where ");

        StringBuffer tmp = null;
        String valueTmp  = null;

        for (int i = 0; i < l; i++) {
            tmp = new StringBuffer();

            valueTmp = "";

            if (TAG_DUE_DATE.equals(fields[i])) {
                tmp.append("\"http://schemas.microsoft.com/mapi/id/{00062003-0000-0000-C000-000000000046}/0x8105\"");

                if (values[i] != null) {
                    tmp.append("=CAST(\"");
                    valueTmp = dateToWebDavTag((java.util.Date)values[i]);
                    tmp.append(valueTmp).append("\" as 'dateTime')");
                } else {
                    tmp.append(" is null ");
                }
            }


            if (TAG_SUBJECT.equals(fields[i])) {
                tmp.append("\"urn:schemas:httpmail:subject\"");

                if (values[i] != null) {
                    tmp.append("='");
                    valueTmp = String.valueOf(values[i]);
                    valueTmp = valueTmp.replaceAll("'","''");
                    valueTmp = StringTools.escapeXml(valueTmp);
                    tmp.append(valueTmp).append("'");
                } else {
                    tmp.append(" is null ");
                }

            }

            if (i != 0 && tmp.length() != 0) {
                clause.append(" and ").append(tmp);
            } else if (tmp.length() != 0) {
                clause.append(tmp);
            }

        }

        clause.append("\r\n");

        return clause.toString();

    }

    /**
     * get tasks from Exchange Server with the given clause
     *
     * @param username
     * @param credentials
     * @param clause
     * @param exchangeFolder
     * @return array of find task
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    private Task[] getTasks(String username,
                            String credentials,
                            String clause,
                            String exchangeFolder) throws DataAccessException {

        Task[] exchangeTasks   = null;

        String response        = null;
        String webDavTaskMsg   = null;

        String webDavHeaderMsg = null;

        String server          = null;
        String resource        = null;

        server   = getServerFromExchangeFolder(exchangeFolder);
        resource = getResourceFromExchangeFolder(exchangeFolder);
        webDavTaskMsg =
                MessageFormat.format(WEBDAV_MSG_SELECT_TASKS,
                                     new Object[] {server, username, resource,
                                     clause});

        webDavHeaderMsg =
                MessageFormat.format(ItemDAO.WEBDAV_HEADER_SELECT,
                                         new Object[] {
                                             "/" +
                                             server +
                                             "/" +
                                             username +
                                             "/" +
                                             resource
                                         }
                                     );

        try {
            response = this.webDavHttp.sendRequest(webDavHeaderMsg, credentials,
                    webDavTaskMsg, FILE_ENCODING);
        } catch (IOException e) {
            throw new DataAccessException(
                    "Error getting Exchange server response",
                    e);
        }

        try {
            int s = getStatusFromResponse(response);
            checkResponseStatus(s);
        } catch (Exception e) {

            throw new DataAccessException("USER " +
                                          username +
                                          " URI " +
                                          exchangeFolder +
                                          " getting exchange task",
                                          e);

        }

        exchangeTasks = getTasksFromWebDavMsg(response);

        return exchangeTasks;
    }

}
