/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package sync4j.exchange.items.task.model;

import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.StringTokenizer;

import sync4j.exchange.items.task.TaskParseException;
import sync4j.exchange.xml.XmlParser;
import sync4j.exchange.xml.XmlParseException;
import sync4j.exchange.util.StringTools;
import java.util.TimeZone;

/**
 * @author Fabio Maggi @ Funambol
 *
 * @version $Id: Task.java,v 1.17 2005/06/27 10:25:08 fabius Exp $
 **/
public class Task {

    //---------------------------------------------------------------- Constants

    private static final String TAG_PP_DATE     = "StartDate";
    private static final String TAG_PP_SUBJECT  = "Subject"  ;
    private static final String TAG_PP_NOTE     = "Body"     ;
    private static final String TAG_PP_COMPLETE = "Complete" ;

    private static final String TAG_PP_CATEGORIES        = "Categories";
    private static final String TAG_PP_DUE_DATE          = "DueDate";
    private static final String TAG_PP_IMPORTANCE        = "Importance";
    private static final String TAG_PP_MILEAGE           = "Mileage";
    private static final String TAG_PP_PERCENT_COMPLETE  = "PercentComplete";
    private static final String TAG_PP_REMINDER_SET      = "ReminderSet";
    private static final String TAG_PP_REMINDER_TIME     = "ReminderTime";
    private static final String TAG_PP_SENSITIVITY       = "Sensitivity";
    private static final String TAG_PP_STATUS            = "Status";

    //------------------------------------------------------------- Private data

    private String id              = null ;
    private String username        = null ;
    private String subject         = null ;
    private String textDescription = null ;
    private String href            = null ;
    private char   state           = ' '  ;
    private boolean complete       = false;

    private String  categories      = null;
    private Date    dueDate         = null;
    private String  importance      = null;
    private String  mileage         = null;
    private String  percentComplete = null;
    private boolean reminderSet     = false;
    private Date    reminderTime    = null;
    private String  sensitivity     = null;
    private String  status          = null;

    private Date   date            = null;
    private Date   createDate      = null;
    private Date   lastModified    = null;

    //------------------------------------------------------------- Constructors

    /**
     * Creates a Task
     *
     */
    public Task() {
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     */
    public Task (String id) {
        this.id = id;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param t client item timestamp
     */
    public Task(String id, Date t) throws TaskParseException {
        this.id           = id;
        this.lastModified = t ;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param state client item state
     */
    public Task(String id, char state) {
        this.id    = id   ;
        this.state = state;
    }

    /**
     * Creates a Task
     *
     * @param id
     * @param href
     * @param state
     * @param lastModified
     */
    public Task(String id          ,
                String href        ,
                char   state       ,
                Date   lastModified) {

        this.id           = id          ;
        this.href         = href        ;
        this.state        = state       ;
        this.lastModified = lastModified;
    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     */
    public Task(String id, String content, Date t) throws TaskParseException {
        getPPTask(id, content, t);
    }

    //----------------------------------------------------------- Public methods
    public String getId() {
        return this.id;
    }

    public String getUsername() {
        return this.username;
    }

    public String getSubject() {
        return this.subject;
    }

    public String getTextDescription() {
        return this.textDescription;
    }

    public Date getDate() {
        return this.date;
    }

    public String getHref() {
        return this.href;
    }

    public Date getLastModified() {
        return this.lastModified;
    }

    public char getState() {
        return this.state;
    }

    public String getCategories() {
        return categories;
    }

    public Date getDueDate() {
        return dueDate;
    }

    public String getImportance() {
        return importance;
    }

    public String getMileage() {
        return mileage;
    }

    public String getPercentComplete() {
        return percentComplete;
    }

    public boolean getReminderSet() {
        return reminderSet;
    }

    public Date getReminderTime() {
        return reminderTime;
    }

    public String getSensitivity() {
        return sensitivity;
    }

    public String getStatus() {
        return status;
    }

    public boolean isComplete() {
        return this.complete;
    }

    public void setId(String id) {
        this.id = id;
    }

    public void setUsername(String username) {
        this.username = username;
    }

    public void setSubject(String subject) {
        this.subject = subject;
    }

    public void setTextDescription(String textDescription) {
        this.textDescription = textDescription;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public void setHref(String href) {
        this.href = href;
    }

    public void setLastModified(Date lastModified) {
        this.lastModified = lastModified;
    }

    public void setState(char state) {
        this.state = state;
    }

    public void setComplete(boolean complete) {
        this.complete = complete;
    }

    public void setCategories(String categories) {
        this.categories = categories;
    }

    public void setDueDate(Date dueDate) {
        this.dueDate = dueDate;
    }

    public void setImportance(String importance) {
        this.importance = importance;
    }

    public void setMileage(String mileage) {
        this.mileage = mileage ;
    }

    public void setPercentComplete(String percentComplete) {
        this.percentComplete = percentComplete ;
    }

    public void setReminderSet(boolean reminderSet) {
        this.reminderSet = reminderSet ;
    }

    public void setReminderTime(Date reminderTime) {
        this.reminderTime = reminderTime;
    }

    public void setSensitivity(String sensitivity) {
        this.sensitivity = sensitivity;
    }

    public void setStatus(String status) {
        this.status = status;
    }

    /**
     * Set a Task properties from Client format content
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     */
    public void getPPTask(String id, String content, Date t)
    throws TaskParseException {

        String dt = null;

        this.id           = id;
        this.lastModified = t ;

        try {

            XmlParser p = new XmlParser();
            dt = p.getXMLInitTagValue(content, TAG_PP_DATE );

            this.date            = getDate(dt);
            this.subject         = p.getXMLInitTagValue(content, TAG_PP_SUBJECT);
            this.textDescription = p.getXMLInitTagValue(content, TAG_PP_NOTE);
            this.categories      = p.getXMLInitTagValue(content, TAG_PP_CATEGORIES);
            this.dueDate         = getDate(p.getXMLInitTagValue(content, TAG_PP_DUE_DATE));
            this.importance      = p.getXMLInitTagValue(content, TAG_PP_IMPORTANCE);
            this.mileage         = p.getXMLInitTagValue(content, TAG_PP_MILEAGE);
            this.percentComplete = p.getXMLInitTagValue(content, TAG_PP_PERCENT_COMPLETE);

            String tmp = p.getXMLInitTagValue(content, TAG_PP_REMINDER_SET);

            this.reminderSet = false;
            if (tmp != null) {
                if (tmp.equalsIgnoreCase("true")) {
                    this.reminderSet = true;
                }
            }

            tmp = p.getXMLInitTagValue(content, TAG_PP_REMINDER_TIME);
            if (tmp != null && !tmp.equals("")) {
                this.reminderTime = p.clientDateToDate(tmp);
            }
            this.sensitivity     = p.getXMLInitTagValue(content, TAG_PP_SENSITIVITY);
            this.status          = p.getXMLInitTagValue(content, TAG_PP_STATUS);


        } catch (NumberFormatException e) {
            throw new TaskParseException(e.getMessage());
        } catch (XmlParseException e) {
            throw new TaskParseException(e.getMessage());
        } catch (Exception e) {
            throw new TaskParseException(e.getMessage());
        }

    }

    /**
     * Set a Task properties to Client format content
     *
     * @return content in client format
     */
    public String toXml() throws TaskParseException {

        StringBuffer xmlMsg = new StringBuffer();
        String date  = null;
        XmlParser p  = new XmlParser();

        try {

            date = p.toClientDate(this.date);
            xmlMsg.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><task>\n");

            if (subject != null) {
                xmlMsg.append("<Subject>");
                xmlMsg.append(subject);
                xmlMsg.append("</Subject>");
            }

            if (textDescription != null) {
                xmlMsg.append("<Body>");
                xmlMsg.append(textDescription);
                xmlMsg.append("</Body>");
            }

            if (date != null && date.length() > 0) {
                xmlMsg.append("<StartDate>");
                xmlMsg.append(date);
                xmlMsg.append("</StartDate>");
            } else {
                xmlMsg.append("<StartDate></StartDate>");
            }

            if (categories != null) {
                xmlMsg.append("<Categories>");
                xmlMsg.append(categories);
                xmlMsg.append("</Categories>");
            }

            if (dueDate != null) {
                xmlMsg.append("<DueDate>");
                xmlMsg.append(XmlParser.toClientDate(dueDate));
                xmlMsg.append("</DueDate>");
            } else {
                xmlMsg.append("<DueDate></DueDate>");
            }

            if (importance != null) {
                xmlMsg.append("<Importance>");
                xmlMsg.append(importance);
                xmlMsg.append("</Importance>");
            }

            if (mileage != null) {
                xmlMsg.append("<Mileage>");
                xmlMsg.append(mileage);
                xmlMsg.append("</Mileage>");
            }

            if (percentComplete != null) {
                xmlMsg.append("<PercentComplete>");
                xmlMsg.append(percentComplete);
                xmlMsg.append("</PercentComplete>");
            }

            xmlMsg.append("<ReminderSet>");
            xmlMsg.append(String.valueOf(reminderSet));
            xmlMsg.append("</ReminderSet>");

            if (reminderTime != null) {
                xmlMsg.append("<ReminderTime>");
                xmlMsg.append(XmlParser.toClientDate(reminderTime));
                xmlMsg.append("</ReminderTime>");
            } else {
                xmlMsg.append("<ReminderTime></ReminderTime>");
            }

            if (sensitivity != null) {
                xmlMsg.append("<Sensitivity>");
                xmlMsg.append(sensitivity);
                xmlMsg.append("</Sensitivity>");
            }

            if (status != null) {
                xmlMsg.append("<Status>");
                xmlMsg.append(status);
                xmlMsg.append("</Status>");
            }

            xmlMsg.append("<Complete>").append(String.valueOf(complete)).append("</Complete>");

            xmlMsg.append("</task>");

         } catch (Exception e) {
             throw new TaskParseException("Error composing task Device: " +
                                           e.getMessage());
         }

         return xmlMsg.toString();
     }

     private Date getDate(String sDate) throws Exception {

          SimpleDateFormat formatter = null;
          String formatUtc = "yyyyMMdd'T'HHmmss'Z'";
          formatter = new SimpleDateFormat(formatUtc);
          formatter.setTimeZone(TimeZone.getTimeZone("UTC"));

          return formatter.parse(sDate);
     }
}
